@@ -1,6 +1,6 @@
 /*
     ac.c - aircraft trace
-    Copyright (C) 2013-2022 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2013-2023 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -14,12 +14,15 @@
 
 */
 
+//mapView.modules.aircraft.instance.aircrafts.326c7192
+
 #include "ac.h"
 
 #include <libzia.h>
 
 #include "header.h"
 
+#include "fifo.h"
 #include "language2.h"
 #include "main.h"
 #include "map.h"
@@ -31,6 +34,8 @@
 #include "subwin.h"
 #include "tsdl.h"
 
+#include "icao2wingspan.inc"
+
 #ifdef HAVE_MATH_H
 #define _USE_MATH_DEFINES // for MSVC
 #endif
@@ -41,44 +46,37 @@ struct its{
 };
 
 
-#define AC_SEP ","
-
 struct acs *gacs;
 
-
 struct acs *init_acs(void){
     struct acs *acs;
     char *wwl;
 
-    if (!cfg->ac_enable) return NULL;
-   
     acs = g_new0(struct acs, 1);
+    progress(VTEXT(T_STARTING_AC));
 
-    if (cfg->ac_url && *cfg->ac_url){
-        acs->url = g_strdup(cfg->ac_url);
-    /*}else{
-        double myw, myh, minw, maxw, minh, maxh;
-        int qtf;
-        char *wwl = cfg->pwwlo;
-        if (ctest) wwl = ctest->pwwlo;
+    acs->dead_minutes = 5;
 
-        minh = maxh = myh = qth(wwl, 0);
-        minw = maxw = myw = qth(wwl, 1);
-        
-        for (qtf = 0; qtf < 360; qtf++){
-            double h2, w2;
-            if (qrbqtf2hw(myh, myw, 500, qtf * M_PI / 180.0, &h2, &w2) != 0) continue;
-            minh = Z_MIN(minh, h2);
-            minw = Z_MIN(minw, w2);
-            maxh = Z_MAX(maxh, h2);
-            maxw = Z_MAX(maxw, w2);
-        }
+    if (!cfg->ac_enable) return NULL;
+    switch (cfg->ac_provider) {
+        case AC_OSN:
+            acs_osn_init(acs);
+            break;
 
-        acs->url = g_strdup_printf("data-live.flightradar24.com/zones/fcgi/feed.js?bounds=%f,%f,%f,%f&faa=1&mlat=1&flarm=1&adsb=1&gnd=1&air=1", maxw * 180.0/M_PI, minw * 180.0/M_PI, minh * 180.0/M_PI, maxh * 180.0/M_PI);*/
+        case AC_FILE:
+            acs_file_init(acs);
+            break;
+
+        case AC_ADSBONE:
+            acs_adsbone_init(acs);
+            break;
+
+        default:
+            log_addf(TRANSLATE("Unknown AS provider"));
+            break;
     }
 
-    
-    progress(VTEXT(T_STARTING_AC));
+
 
 #ifdef Z_HAVE_SDL
     if (sdl){
@@ -90,10 +88,11 @@ struct acs *init_acs(void){
     if (ctest) wwl = ctest->pwwlo;
     acs->sh.myh = qth(wwl, 0);
     acs->sh.myw = qth(wwl, 1);
+    acs->sh.filename = g_strdup(cfg->ac_filename);
 
     init_acs_infos(acs);
-
-    acs->http_timer = zselect_timer_new(zsel, 1000, acs_http_timer, acs);
+    acs->unknownids = g_hash_table_new(g_int_hash, g_int_equal);
+    MUTEX_INIT(acs->unknownids);
     MUTEX_INIT(acs->sh);
     acs->thread = g_thread_try_new("ac", acs_thread, (gpointer)acs, NULL);
     return acs;
@@ -107,9 +106,15 @@ void free_acs(struct acs *acs){
         acs->thread_break = 1;
         g_thread_join(acs->thread);
     }
-    if (acs->http_timer) zselect_timer_kill(zsel, acs->http_timer);
-    zhttp_free(acs->http);
-    g_free(acs->url);
+
+    MUTEX_FREE(acs->unknownids);
+    g_hash_table_destroy(acs->unknownids);
+
+    acs_osn_free(acs);
+    acs_file_free(acs);
+    acs_adsbone_free(acs);
+
+    zg_free0(acs->sh.filename);
     g_free(acs->infolocqso.locator);
     zg_ptr_array_free_all(acs->sh.acs);
     free_acs_infos(acs);
@@ -117,143 +122,6 @@ void free_acs(struct acs *acs){
     g_free(acs);
 }
 
-// feed since 1/2015 http://arn.data.fr24.com/zones/fcgi/feed.js?bounds=55,44,4,24&faa=1&mlat=1&flarm=1&adsb=1&gnd=1&air=1
-// feed since 4/2016 http://data-live.flightradar24.com/zones/fcgi/feed.js?bounds=55,44,4,24&faa=1&mlat=1&flarm=1&adsb=1&gnd=1&air=1
-
-// thread
-void acs_load(struct acs *acs, const char *data){
-    char *c, *token_ptr = NULL;
-    int maxasl = 0;
-    char *d;
-
-    d = g_strdup(data);
-
-    // until 2015 "15aec22":["4B5C8C",46.2370,6.1076,45,0,32,"0000","T-LSGG2","","-",1370602183,"","","",1,0,"HERON2",0],"15a
-
-    // {"full_count":10175,"version":4,"5bbca6b":["",52.2627,11.7722,24,45000,447,"5707","F-EDDC2","F2TH","",1426164599,"","","",0,-128,"F2TH",0]
-    // ,"5bbc830":["4007F3",44.7224,4.7571,350,43000,454,"4010","F-LFBO7","B772","G-YMMH",1426164598,"ABV","LHR","BA82",0,0,"BAW82",0]
-
-    token_ptr = NULL;
-    c = strtok_r(d, "[", &token_ptr);
-    while (1){
-        struct ac *ac;
-        //dbg("ac='%s'\n", gs->str);
-        ac = g_new0(struct ac, 1);
-        c = strtok_r(NULL, "[", &token_ptr);
-        if (!c) goto x;
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // "C87EE8"
-        if (!c) goto x;
-        if (!*c) goto x;
-		g_strlcpy(ac->id, c+1, AC_ID_MAXLEN);
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // latitude
-        if (!c) goto x;
-        ac->w = atof(c) * M_PI / 180.0;
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // longitude
-        if (!c) goto x;
-        ac->h = atof(c) * M_PI / 180.0;
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // azimuth in degrees
-        if (!c) goto x;
-        ac->qtfdir = (atoi(c) - 90) * M_PI / 180.0;
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr);  // asl in feets
-        if (!c) goto x;
-        ac->asl = atoi(c) * 0.3048;
-        if (ac->asl < cfg->ac_minalt){ // unusable altitude
-            g_free(ac);
-            continue;
-        }
-
-        //if (ac->asl < 2000) goto abrt;
-        if (ac->asl > maxasl) maxasl = ac->asl;
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // speed in knots
-        if (!c) goto x;
-        ac->speed = atoi(c) * 1.852;
-        //if (ac->speed < 100) goto abrt;
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // "0000"
-        if (!c) goto x;
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // "T-LSFF2"
-        if (!c) goto x;
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // "B772"
-        if (!c) goto x;
-        if (*c == '"') c++;
-        if (strlen(c) > 0 && c[strlen(c) - 1] == '"') c[strlen(c) - 1] = '\0';
-        g_strlcpy(ac->icao, c, AC_ICAO_MAXLEN);
-        if (strlen(ac->icao) > 2 && acs_get_wingspan(acs, ac->icao) == 0) dbg("Unknown ICAO %s\n", ac->icao);
-
-
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // "G-YMMH"
-        if (!c) goto x;
-        
-        c = strtok_r(NULL, AC_SEP, &token_ptr); // timestamp
-        if (!c) goto x;
-        ac->when = (time_t)atol(c);
-        g_ptr_array_add(acs->th.acs, ac);
-        continue;
-x:;        
-        g_free(ac);
-        break;
-    }
-    g_free(d);
-    acs->validity = time(NULL) + 10 * 60;
-}
-// thread
-void acs_load2(struct acs *acs, const char *data){
-    GString *gs;
-    char *c, *token_ptr = NULL;
-    int  maxasl = 0, dlen;
-    long pos = 0;
-
-
-    dlen = strlen(data);
-    gs = g_string_sized_new(50);
-    while (zfile_mgets(gs, data, &pos, dlen, 0)){
-        struct ac *ac;
-        //dbg("ac='%s'\n", gs->str);
-        ac = g_new0(struct ac, 1);
-        if (gs->len == 0) break; // TODO
-        c = strtok_r(gs->str, AC_SEP, &token_ptr);
-        if (!c) goto x;
-        ac->w = atof(c) * M_PI / 180.0;
-        c = strtok_r(NULL, AC_SEP, &token_ptr);
-        if (!c) goto x;
-        ac->h = atof(c) * M_PI / 180.0;
-        c = strtok_r(NULL, AC_SEP, &token_ptr);
-        if (!c) goto x;
-        ac->qtfdir = (atoi(c) - 90) * M_PI / 180.0;
-        c = strtok_r(NULL, AC_SEP, &token_ptr);
-        if (!c) goto x;
-        ac->asl = atoi(c) * 0.3048;
-        //if (ac->asl < 2000) goto abrt;
-        if (ac->asl > maxasl) maxasl = ac->asl;
-        c = strtok_r(NULL, AC_SEP, &token_ptr);
-        if (!c) goto x;
-        ac->speed = atoi(c) * 1.852;
-        //if (ac->speed < 100) goto abrt;
-        c = strtok_r(NULL, AC_SEP, &token_ptr);
-        if (!c) goto x;
-        ac->when = (time_t)atol(c);
-        
-        c = strtok_r(NULL, AC_SEP, &token_ptr);
-        if (c != NULL){
-            g_strlcpy(ac->icao, c, AC_ICAO_MAXLEN);
-        }
-
-        g_ptr_array_add(acs->th.acs, ac);
-        continue;
-x:;        
-        g_free(ac);
-        break;
-    }
-    g_string_free(gs, TRUE);
-    acs->validity = time(NULL) + 10 * 60;
-}
 
 
 // thread
@@ -271,24 +139,23 @@ void ac_compute(struct acs *acs, struct ac *ac, struct ac_counterpart *ctp, stru
     
     now = time(NULL);
     dt = now - ac->when;
-    if (dt > 5 * 60){
-        res->crossing = 0;
-        res->start = (time_t)0;
-        return;
-    }
+    //dbg("dt=%d\n", (int)dt);
+    ac->flyingdutchman = (dt > gacs->dead_minutes * 60);
     qrbqtf2hw(ac->h, ac->w, ac->speed * dt / 3600.0, ac->qtfdir + (90 * M_PI / 180.0), &h, &w);
     hw2qrbqtf(acs->th.myh, acs->th.myw, h, w, &qrb, &ac->qtf); 
-	ac->kx = (int)round(qrb * sin(ac->qtf));
-	ac->ky = (int)round(-qrb * cos(ac->qtf));
+    ac->kx = (int)round(qrb * sin(ac->qtf));
+    ac->ky = (int)round(-qrb * cos(ac->qtf));
    // dbg("%5.3f %5.3f -> %5.3f %5.3f = %f %f %f | %d %d\n", mh, mw, ac->h, ac->w, qrb, qtfrad, (qtfrad * 180/M_PI + 90), ac->kx, ac->ky);
     //dbg("qtf=%f %f speed=%d\n", ac->qtf, ac->qtf * 180.0 / M_PI, ac->speed);
     
-	qrbqtf2hw(h, w, ac->speed * 10.0 / 60.0, ac->qtfdir + (90 * M_PI / 180.0), &h2, &w2);
-	hw2qrbqtf(acs->th.myh, acs->th.myw, h2, w2, &qrb, &qtf);
-	ac->pakx = (int)round(qrb * sin(qtf));
-	ac->paky = (int)round(-qrb * cos(qtf));
+    qrbqtf2hw(h, w, ac->speed * 10.0 / 60.0, ac->qtfdir + (90 * M_PI / 180.0), &h2, &w2);
+    hw2qrbqtf(acs->th.myh, acs->th.myw, h2, w2, &qrb, &qtf);
+    ac->pakx = (int)round(qrb * sin(qtf));
+    ac->paky = (int)round(-qrb * cos(qtf));
     //dbg("h=%f  w=%f | kx=%d  ky=%d \n", h * 180.0/M_PI, w * 180.0/M_PI, ac->kx[i], ac->ky[i]);
 
+    ac->is_computed = 1;
+
 #ifdef Z_HAVE_SDL
     ii = 0;
     retin = z_point_is_in_quadrangle(ac->kx, ac->ky, ctp->arkx[0], ctp->arky[0], ctp->arkx[1], ctp->arky[1], ctp->arkx[2], ctp->arky[2], ctp->arkx[3], ctp->arky[3]);
@@ -351,7 +218,7 @@ void ac_compute(struct acs *acs, struct ac *ac, struct ac_counterpart *ctp, stru
     res->stop += now;
 
     res->now = now;
-	res->angle = ctp->qtfrad - (ac->qtfdir + M_PI / 2);
+    res->angle = ctp->qtfrad - (ac->qtfdir + M_PI / 2);
     while (res->angle < 0.0) res->angle += 2 * M_PI;
     while (res->angle > 2 * M_PI) res->angle -= 2 * M_PI;
     if (res->angle > M_PI) res->angle = res->angle - M_PI;
@@ -484,10 +351,13 @@ void plot_ac(struct subwin *sw, struct ac *ac){
     struct ac_counterpart *ctp = sw->type == SWT_KST ? &gacs->infoqso : &gacs->tmpqso;
     struct ac_result *res = sw->type == SWT_KST ? &ac->infores : &ac->tmpres;
 
+    if (!ac->is_computed) return;
+
     //todo outline
     km2px_d(sw, ac->kx, ac->ky, &px, &py);
-    r = acs_get_wingspan(gacs, ac->icao) / 8;
-    if (r == 0) r = 10;
+    //r = acs_get_wingspan(gacs, ac->icao) / 8;
+    r = ac->wingspan / 8;
+    if (r == 0) r = 16; 
     if (sw->zoom < 10000) r = (r * sw->zoom) / 10000;
     
     //dbg("qtf=%f %f \n", ac->qtf, ac->qtf * 180.0 / M_PI);
@@ -510,10 +380,10 @@ void plot_ac(struct subwin *sw, struct ac *ac){
     if (undercursor){
         fcolor = z_makecol(21, 193, 106);
         bcolor = z_makecol(15, 150, 70);
-	}else if (ac->id && *ac->id && strcmp(ac->id, gacs->sh.tracked) == 0){
-		fcolor = z_makecol(255, 0, 255);
-		bcolor = z_makecol(192, 0, 192);
-	}else if (ctp->path_valid == 1 && res->under){
+    }else if (ac->id && *ac->id && strcmp(ac->id, gacs->sh.tracked) == 0){
+        fcolor = z_makecol(255, 0, 255);
+        bcolor = z_makecol(192, 0, 192);
+    }else if (ctp->path_valid == 1 && res->under){
         fcolor = z_makecol(128, 128, 128);
         bcolor = z_makecol(64, 64, 64);
     }else if (ctp->path_valid == 1 && res->crossing){
@@ -523,12 +393,21 @@ void plot_ac(struct subwin *sw, struct ac *ac){
         fcolor = z_makecol(192, 192, 192);
         bcolor = z_makecol(128, 128, 128);
     }
-    
-    z_triangle(sw->screen, vx, vy, vx1, vy1, vx2, vy2, bcolor);
-    z_line(sw->screen, vx, vy, vx1, vy1, fcolor);
-    z_line(sw->screen, vx, vy, vx2, vy2, fcolor);
-    z_line(sw->screen, vx1, vy1, vx2, vy2, fcolor);
-    z_putpixel(sw->screen, px, py, fcolor);
+   
+    if (ac->flyingdutchman) {
+        z_do_line(sw->screen, vx, vy, vx1, vy1, fcolor, z_dotfce);
+        z_do_line(sw->screen, vx, vy, vx2, vy2, fcolor, z_dotfce);
+        z_do_line(sw->screen, vx1, vy1, vx2, vy2, fcolor, z_dotfce);
+    }
+    else {
+        if (ac->wingspan > 0) {
+            z_triangle(sw->screen, vx, vy, vx1, vy1, vx2, vy2, bcolor);
+        }
+        z_line(sw->screen, vx, vy, vx1, vy1, fcolor);
+        z_line(sw->screen, vx, vy, vx2, vy2, fcolor);
+        z_line(sw->screen, vx1, vy1, vx2, vy2, fcolor);
+        z_putpixel(sw->screen, px, py, fcolor);
+    }
 
     vx  = (int)round(px + 0.6*r * cos(rad));
     vy  = (int)round(py + 0.6*r * sin(rad));
@@ -595,10 +474,34 @@ void plot_info_ac(struct subwin *sw, SDL_Surface *surface, struct ac *ac){
         sw->ac_plotinfo = 1;
         color=z_makecol(21, 193, 106);
 
-        zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, VTEXT(T_ASL_5D_M), ac->asl); y    +=FONT_H+4;
-        zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, VTEXT(T_SPEED4D_KMH), ac->speed); y += FONT_H + 4;
-        zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("Plane: %s")/*VTEXT(T_SPEED4D_KMH)*/, ac->icao); y += FONT_H + 4;
-        zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("WingS: %d ft")/*VTEXT(T_SPEED4D_KMH)*/, acs_get_wingspan(gacs, ac->icao)); y += FONT_H + 4;
+        if (cfg->ac_imperial){
+            zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, "ASL: %5d ft", ac->feets); y += FONT_H + 4;
+            zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("Speed: %3d kt"), ac->knots); y += FONT_H + 4;
+        }else{
+            zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, VTEXT(T_ASL_5D_M), ac->asl); y += FONT_H + 4;
+            zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, VTEXT(T_SPEED4D_KMH), ac->speed); y += FONT_H + 4;
+        }
+
+        if (ac->icaotype != NULL && strlen(ac->icaotype) > 0) {
+            zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("Type:  %s")/*VTEXT(T_SPEED4D_KMH)*/, ac->icaotype); y += FONT_H + 4;
+        }
+        if (ac->callsign != NULL && strlen(ac->callsign) > 0) {
+            zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("Call:  %s")/*VTEXT(T_SPEED4D_KMH)*/, ac->callsign); y += FONT_H + 4;
+        }
+        if (ac->wingspan == 0) {
+            zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("WingS: unknown")); y += FONT_H + 4;
+        }else{
+            if (cfg->ac_imperial) {
+                zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("WingS: %d ft"), ac->wingspan); y += FONT_H + 4;
+            }else{
+                zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("WingS: %d m"), (int)round(ac->wingspan * 0.3048)); y += FONT_H + 4;
+            }
+        }
+        time_t upd = time(NULL) - ac->when;
+        zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("Updated: %d min"), upd / 60); y += FONT_H + 4;
+        //zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("%3.1f %3.1f"), ac->w*180.0/M_PI, ac->h * 180.0 / M_PI); y += FONT_H + 4;
+        //zsdl_printf(sw->screen, x, y, color, 0, ZFONT_TRANSP, TRANSLATE("%3.1f %3.1f"), ac->kx, ac->ky); y += FONT_H + 4;
+
 
         res = sw->type == SWT_KST ? &ac->infores : &ac->tmpres;
         if (res->start != (time_t)0){
@@ -642,21 +545,6 @@ void acs_redraw(struct subwin *sw, struct acs *acs){
 }
 #endif
 
-/*
-static void ac_fill_band_ac_k(struct band *b){
-    
-    
-    if (b->qrg_min < 200000) b->ac_k = 1.37;
-    else if (b->qrg_min < 500000) b->ac_k = 0.92;
-    else if (b->qrg_min < 1400000) b->ac_k = 0.8;
-    else if (b->qrg_min < 3000000) b->ac_k = 0.77;
-    else if (b->qrg_min < 4000000) b->ac_k = 0.75;
-    else if (b->qrg_min < 6000000) b->ac_k = 0.72;
-    else b->ac_k = 0.7;   
-    b->ac_k = 1;
-}
-*/
-
 
 double acs_min_asl(struct acs *acs, double qrb, struct band *b, double elev_rad){
     double k = 1.0;
@@ -688,6 +576,7 @@ double acs_max_qrb(struct acs *acs, double asl, struct band *b, double elev_rad)
     //return qrb;
 }
 
+#if 0
 void acs_http_timer(void *arg){
     struct acs *acs = (struct acs *)arg;
 
@@ -703,12 +592,17 @@ void acs_http_timer(void *arg){
         char *url = g_strdup_printf("tucnak.vaiz.cz/acurl.php?mycall=%s&mywwl=%s", ctest ? ctest->pcall : cfg->pcall, ctest ? ctest->pwwlo : cfg->pwwlo);
         zhttp_get(acs->http, zsel, url, acs_goturl, acs);
         g_free(url);
+    /* } else if (strcmp(acs->url, "planefinder") == 0) {
+        char* url = g_strdup_printf("https://data-live.planefinder.net/api/aircraft/live/bounds/27,-43,63,34/%d", time(NULL));
+        dbg("url='%s'\n", url);
+        zhttp_get(acs->http, zsel, url, acs_downloaded_callback, acs);
+        g_free(url);*/
     }else{
         zhttp_get(acs->http, zsel, acs->url, acs_downloaded_callback, acs);
     }
     acs->http_timer = zselect_timer_new(zsel, 50*1000, acs_http_timer, acs); // restart when freezes
 
-    dbg("acs_timer: zhttp_get\n");
+    //dbg("acs_timer: zhttp_get\n");
 }
 
 void acs_goturl(struct zhttp *http){
@@ -716,28 +610,28 @@ void acs_goturl(struct zhttp *http){
     int remains = 10000;
     struct acs *acs = (struct acs *)http->arg;
 
-    dbg("acs_goturl\n");
+    //dbg("acs_goturl\n");
 
     if (acs->http_timer){
         zselect_timer_kill(zsel, acs->http_timer);
         acs->http_timer = 0;
     }
 
-	if (http_is_content_type(http, "text/plain;charset=UTF-8")){
-		data = g_new(char, http->response->len + 1);
-		zbinbuf_getstr(http->response, http->dataofs, data, http->response->len + 1);
+    if (http_is_content_type(http, "text/plain;charset=UTF-8")){
+        data = g_new(char, http->response->len + 1);
+        zbinbuf_getstr(http->response, http->dataofs, data, http->response->len + 1);
 
-		if (acs->http->errorstr == NULL){
-			z_split2(data, ';', &status, &url, 0);
-			if (status != NULL && status[0] == 'A'){
-				remains = 1;
-				acs->url = g_strdup(url);
-				g_free(status);
-				g_free(url);
-			}
-		}
-		g_free(data);
-	}
+        if (acs->http->errorstr == NULL){
+            z_split2(data, ';', &status, &url, 0);
+            if (status != NULL && status[0] == 'A'){
+                remains = 1;
+                acs->url = g_strdup(url);
+                g_free(status);
+                g_free(url);
+            }
+        }
+        g_free(data);
+    }
     zhttp_free(http);
     acs->http = NULL;
 
@@ -750,30 +644,39 @@ void acs_downloaded_callback(struct zhttp *http){
     int remains;
     struct acs *acs = (struct acs *)http->arg;
 
-    dbg("acs_downloaded_callback\n");
+    //dbg("acs_downloaded_callback\n");
 
     if (acs->http_timer){
         zselect_timer_kill(zsel, acs->http_timer);
         acs->http_timer = 0;
     }
 
-	if (http_is_content_type(http, "application/json") || http_is_content_type(http, "text/plain;charset=UTF-8")){
-		data = g_new(char, http->response->len + 1);
-		zbinbuf_getstr(http->response, http->dataofs, data, http->response->len + 1);
-		zhttp_free(http);
-		acs->http = NULL;
+    if (http->state == ZHTTPST_ERROR && http->errorstr){
+        dbg("ERROR acs_downloaded_callback: %s\n", http->errorstr);
+        zhttp_free(http);
+        acs->http = NULL;
+    }else{
+        if (http_is_content_type(http, "application/json") || 
+            http_is_content_type(http, "text/plain;charset=UTF-8") /* ||
+            strcmp(acs->url, "planefinder") == 0 // any*/
+           ){
+            data = g_new(char, http->response->len + 1);
+            zbinbuf_getstr(http->response, http->dataofs, data, http->response->len + 1);
+            zhttp_free(http);
+            acs->http = NULL;
 
-		MUTEX_LOCK(acs->sh);
-		g_free(acs->sh.data);
-		acs->sh.data = data;
-		MUTEX_UNLOCK(acs->sh);
-	}
+            MUTEX_LOCK(acs->sh);
+            g_free(acs->sh.data);
+            acs->sh.data = data;
+            MUTEX_UNLOCK(acs->sh);
+        }
+    }
 
     gettimeofday(&tv, NULL);
     remains = 60000 - ((tv.tv_sec % 60) * 1000 + tv.tv_usec/1000);
     acs->http_timer = zselect_timer_new(zsel, remains, acs_http_timer, acs);
 }
-
+#endif
 gpointer acs_thread(gpointer arg){
     struct acs *acs = (struct acs *)arg;
     int new_tmpctp = 0;
@@ -781,11 +684,12 @@ gpointer acs_thread(gpointer arg){
 
     zg_thread_set_name("Tucnak AS");
     while (!acs->thread_break){
-        char *data;
+        //char *data;
+        //int data_len;
         struct ac *ac;
         int i, j;
-		char tracked[AC_ID_MAXLEN];
-		int rotnr;
+        char tracked[AC_ID_MAXLEN];
+        int rotnr;
 
 
         for (i = 0; i < 10; i++) {
@@ -798,19 +702,27 @@ gpointer acs_thread(gpointer arg){
         //dbg("------acs_thread--------\n");
 
         MUTEX_LOCK(acs->sh);
-        if (acs->sh.data){
-            data = acs->sh.data;
+        acs->th.myh = acs->sh.myh;
+        acs->th.myw = acs->sh.myw;
+        MUTEX_UNLOCK(acs->sh);
+
+        if (acs->load_from_thread) acs->load_from_thread(acs);
+        //acs_read_feed(acs);
+
+        MUTEX_LOCK(acs->sh);
+     
+        if (acs->sh.data){ // take data from main thread (http)
+            char *data = acs->sh.data;
+          //  data_len = acs->sh.data_len;
             acs->sh.data = NULL;
             acs->th.myh = acs->sh.myh;
             acs->th.myw = acs->sh.myw;
             MUTEX_UNLOCK(acs->sh);
         
             acs->th.acs = g_ptr_array_new();
-            if (strncmp(data, "{\"full_count\":", 14) == 0){
-                acs_load(acs, data);
-            }else{
-                acs_load2(acs, data);
-            }
+
+            if (acs->load_from_main_thread) acs->load_from_main_thread(acs, data);
+
             g_free(data);
             MUTEX_LOCK(acs->sh);
             zg_ptr_array_free_all(acs->sh.acs);
@@ -829,8 +741,8 @@ gpointer acs_thread(gpointer arg){
             acs->sh.new_infoctp = 0;
             new_infoctp = 1;
         }
-		g_strlcpy(tracked, acs->sh.tracked, AC_ID_MAXLEN);
-		rotnr = acs->sh.rotnr;
+        g_strlcpy(tracked, acs->sh.tracked, AC_ID_MAXLEN);
+        rotnr = acs->sh.rotnr;
         MUTEX_UNLOCK(acs->sh);
 
         if (acs->sh.acs){
@@ -843,15 +755,15 @@ gpointer acs_thread(gpointer arg){
                 new_infoctp = 0;
             }
             //ST_START();
-			int qtf = -360;
+            int qtf = -360;
             for (i = 0; i < acs->sh.acs->len; i++){
                 ac = (struct ac *)g_ptr_array_index(acs->sh.acs, i);
                 ac_compute(acs, ac, &acs->tmpqso, &ac->tmpres);
-				if (strlen(ac->id) > 0 && strcmp(ac->id, tracked) == 0){
-					qtf = (int)round(ac->qtf * 180.0 / M_PI);
-					dbg("qtf=%d\n", qtf);
-					//if (qtf >= 360) qtf -= 360;
-				}
+                if (strlen(ac->id) > 0 && strcmp(ac->id, tracked) == 0){
+                    qtf = (int)round(ac->qtf * 180.0 / M_PI);
+                    //dbg("qtf=%d\n", qtf);
+                    //if (qtf >= 360) qtf -= 360;
+                }
                 if (acs->thread_break) break;
             }
             for (i = 0; i < acs->sh.acs->len; i++){
@@ -922,23 +834,20 @@ next:;
                 if (acs->thread_break) break;
             }
 
-			if (qtf > -360){
-				MUTEX_LOCK(acs->sh);
+            if (qtf > -360){
+                MUTEX_LOCK(acs->sh);
 #define HYSTERESIS 1
-				if (strlen(acs->sh.tracked) > 0){ // prevent race condition when ac_track(NULL) was called during for { ac_compute }
-					struct rotar *rot = get_rotar(rotnr);
-					if (rot->type != ROT_REMOTE /* not sure if thread-safe */) {
-						//dbg("rot_seek(%d, %d)\n", rotnr, qtf);
-						if (abs(rot->qtf - qtf) > HYSTERESIS){
-							rot_seek(rot, qtf, -90);
-						}
-					}
-				}
-				MUTEX_UNLOCK(acs->sh);
-
-			}
-			//ST_STOP("ac_recompute");
-            zselect_msg_send(zsel, "AC;r"); 
+                if (strlen(acs->sh.tracked) > 0){ // prevent race condition when ac_track(NULL) was called during for { ac_compute }
+                    struct rotar *rot = get_rotar(rotnr);
+                        //dbg("rot_seek(%d, %d)\n", rotnr, qtf);
+                    if (abs(rot->qtf - qtf) > HYSTERESIS){
+                        rot_seek(rot, qtf, -90);
+                    }
+                }
+                MUTEX_UNLOCK(acs->sh);
+            }
+            //ST_STOP("ac_recompute");
+            zselect_msg_send(zsel, "AC;r");
         }
 
     }
@@ -947,7 +856,7 @@ next:;
 
 void acs_update_qth(struct acs *acs){
     if (!acs) return;
-	if (!gses) return;
+    if (!gses) return;
 
     MUTEX_LOCK(acs->sh);
     if (acs->sh.myh != gses->myh || acs->sh.myw != gses->myw){
@@ -1066,7 +975,7 @@ void init_acs_infos(struct acs *acs){
 
     acs->infos = g_ptr_array_new();
     while (1){
-		struct ac_info *aci;
+        struct ac_info *aci;
 
         ic = strtok_r(buf2, " ", &token_ptr);
         wingspan = strtok_r(NULL, "\n", &token_ptr);
@@ -1076,7 +985,7 @@ void init_acs_infos(struct acs *acs){
         if (ic == NULL || wingspan == NULL) continue;
         
         aci = g_new0(struct ac_info, 1);
-        g_strlcpy(aci->icao, ic, AC_ICAO_MAXLEN);
+        g_strlcpy(aci->icaotype, ic, AC_ICAOTYPE_MAXLEN);
         aci->wingspan = atoi(wingspan);
         g_ptr_array_add(acs->infos, aci);
     }
@@ -1090,53 +999,77 @@ void free_acs_infos(struct acs *acs){
     acs->infos = NULL;
 }
     
-int acs_get_wingspan(struct acs *acs, char *icao){
-    int i;
+//int acs_get_wingspan(struct acs *acs, char *icao){
+//    int i;
+//
+//    for (i = 0; i < acs->infos->len; i++){
+//        struct ac_info *aci = (struct ac_info *)g_ptr_array_index(acs->infos, i);
+//        if (strcmp(aci->icao, icao) == 0) return aci->wingspan;
+//    }
+//    return 0;
+//}
 
+void ac_fill_wingspan(struct acs *acs, struct ac* ac) {
+    uint32_t id = strtoul(ac->id, NULL, 16);
+
+    for (const uint32_t* u = icao2wingspan; *u != 0; u++) {
+        if ((*u >> 8) == id) {
+            ac->wingspan = *u & 0xff;
+            return;
+        }
+    }
+
+    int i;
+     
     for (i = 0; i < acs->infos->len; i++){
         struct ac_info *aci = (struct ac_info *)g_ptr_array_index(acs->infos, i);
-        if (strcmp(aci->icao, icao) == 0) return aci->wingspan;
+        if (strcmp(aci->icaotype, ac->icaotype) == 0) ac->wingspan = aci->wingspan;
     }
-    return 0;
+
+     /*gpointer p = g_hash_table_lookup(acs->unknownids, GINT_TO_POINTER(id));
+     if (p == NULL) {
+         g_hash_table_insert(acs->unknownids, GINT_TO_POINTER(id), NULL);
+         dbg("unknownids insert %06X\n", id);
+     }*/
 }
 
 #ifdef Z_HAVE_SDL
 struct ac *ac_under(struct acs *acs, struct subwin *sw){
-	struct ac *ac, *ret = NULL;
-	int i;
+    struct ac *ac, *ret = NULL;
+    int i;
 
-	if (!gacs) return NULL;
+    if (!gacs) return NULL;
 
-	MUTEX_LOCK(acs->sh);
-	if (acs->sh.acs){
-		for (i = 0; i < acs->sh.acs->len; i++){
-			ac = (struct ac *)g_ptr_array_index(acs->sh.acs, i);
-			ac_zorder(sw, ac);
-			if (ac->zorder == ACZO_CURSOR){
-				ret = ac;
-				break;
-			}
-		}
-	}
-	MUTEX_UNLOCK(acs->sh);
-	return ret;
+    MUTEX_LOCK(acs->sh);
+    if (acs->sh.acs){
+        for (i = 0; i < acs->sh.acs->len; i++){
+            ac = (struct ac *)g_ptr_array_index(acs->sh.acs, i);
+            ac_zorder(sw, ac);
+            if (ac->zorder == ACZO_CURSOR){
+                ret = ac;
+                break;
+            }
+        }
+    }
+    MUTEX_UNLOCK(acs->sh);
+    return ret;
 }
 #endif
 
 void ac_track(struct acs *acs, char *id, int rotnr){
-	if (!acs) return;
+    if (!acs) return;
 
-	MUTEX_LOCK(acs->sh);
+    MUTEX_LOCK(acs->sh);
 
-	if (id == NULL){
-		g_strlcpy(acs->sh.tracked, "", AC_ID_MAXLEN);
-		acs->sh.rotnr = -1;
-	}else{
-		g_strlcpy(acs->sh.tracked, id, AC_ID_MAXLEN);
-		acs->sh.rotnr = rotnr;
-	}
+    if (id == NULL){
+        g_strlcpy(acs->sh.tracked, "", AC_ID_MAXLEN);
+        acs->sh.rotnr = -1;
+    }else{
+        g_strlcpy(acs->sh.tracked, id, AC_ID_MAXLEN);
+        acs->sh.rotnr = rotnr;
+    }
 
-	MUTEX_UNLOCK(acs->sh);
+    MUTEX_UNLOCK(acs->sh);
 }
 
 
