@@ -1,6 +1,6 @@
 /*
     ac.h - aircraft trace
-    Copyright (C) 2012 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2012-2023 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -22,6 +22,12 @@ struct qso;
 struct zhttp;
 struct qrv_item;
 
+enum acs_provider {
+	AC_OSN,
+	AC_FILE,
+	AC_ADSBONE
+};
+
 struct ac_result{
 	int crossing; // next valid only when crossing == 1
     int under;
@@ -38,30 +44,37 @@ struct ac_result{
 #define ACZO_CURSOR 3
 
 #define AC_ID_MAXLEN 7
-#define AC_ICAO_MAXLEN 5
-
+#define AC_ICAOTYPE_MAXLEN 7
+#define AC_CALLSIGN_MAXLEN 10 // FAA/NAS limit is 7 characters
 
 struct ac{
 	char id[AC_ID_MAXLEN]; // ICAO 24-bit address
-	double w, h;
+	char callsign[AC_CALLSIGN_MAXLEN];
+	double w, h; // radians
 	double qtfdir; // radians
-    int asl; // meters
-    int speed; // km/h
+	int feets;
+	int asl; // meters
+	int knots;
+	int speed; // km/h
+	int icon;
 	time_t when; 
 	
     // state
+	int is_computed;
     double kx, ky; // actual position
 	double pakx, paky; // predicted path after 10 minutes
 	double qtf; // from me
+	int flyingdutchman; // data is too old
 
 	struct ac_result tmpres, infores, qrvres; 
 	int zorder;
-	gchar icao[AC_ICAO_MAXLEN];
+	gchar icaotype[AC_ICAOTYPE_MAXLEN];
 	int wingspan; // feets
 };
 
 struct acs_shared{
 	char *data; // http in main thread -> acs_thread
+	char *filename;
 	GPtrArray *acs;
 	double myh, myw;
 
@@ -71,7 +84,7 @@ struct acs_shared{
     double infoctpqrb, infoctpqtf;
     int new_infoctp;
 
-	char tracked[AC_ID_MAXLEN];
+	char tracked[AC_ICAOTYPE_MAXLEN];
 	int rotnr;
 };
 
@@ -93,16 +106,28 @@ struct ac_counterpart{
 struct acs{
 	// only main thread
 //	struct subwin *sw;
-	char *url;
     int normal, active;
 	GThread *thread;
-	int http_timer;
-	struct zhttp *http;
 	struct qso infolocqso;
     time_t validity;
 
+	int osn_http_timer;
+	struct zhttp *osn_http;
+	int osn_requests_remaining;
+	char* osn_http_error;
+
+	int adsbone_http_timer;
+	struct zhttp* adsbone_http;
+	char* adsbone_http_error;
+
+
+#ifdef Z_HAVE_SDL
+	void (*plot_info)(struct subwin* sw, SDL_Rect* area, struct acs* acs);
+#endif
+
 	// shared without lock
 	int thread_break;
+	int dead_minutes;
 
     struct ac_counterpart tmpqso;// TODO -> th
     struct ac_counterpart infoqso;// TODO -> th
@@ -115,10 +140,14 @@ struct acs{
 	// only thread
 	struct acs_thread th;
 	GPtrArray *infos; // of ac_info
+	void (*load_from_main_thread)(struct acs* acs, const char* data);
+	void (*load_from_thread)(struct acs* acs);
+	GHashTable* unknownids;
+	MUTEX_DEFINE(unknownids);
 };
 
 struct ac_info{
-	gchar icao[5];
+	gchar icaotype[AC_ICAOTYPE_MAXLEN];
 	int wingspan;
 };
 
@@ -126,8 +155,6 @@ extern struct acs *gacs;
 
 struct acs *init_acs(void);
 void free_acs(struct acs *acs);
-void acs_load(struct acs *acs, const char *data); // called in thread
-void acs_load2(struct acs *acs, const char *data); // called in thread
 void ac_compute(struct acs *acs, struct ac *ac, struct ac_counterpart *ctp, struct ac_result *res); // called in thread
 #ifdef Z_HAVE_SDL
 void plot_path(struct subwin *sw, SDL_Surface *surface, struct band *band, struct qso *qso, struct ac_counterpart *ctp);
@@ -140,9 +167,7 @@ void ac_redraw(void);
 
 double acs_min_asl(struct acs *acs, double qrb, struct band *b, double elev_rad);
 double acs_max_qrb(struct acs *acs, double asl, struct band *b, double elev_rad);
-void acs_http_timer(void *arg);
-void acs_goturl(struct zhttp *http);
-void acs_downloaded_callback(struct zhttp *http);
+//void acs_goturl(struct zhttp *http);
 gpointer acs_thread(gpointer arg);
 void acs_update_qth(struct acs *acs);
 void ac_update_tmpctp(char *wwl);
@@ -151,8 +176,35 @@ void ac_format(struct qrv_item *qi, char *acstart, char *acint, int flags);
 
 void init_acs_infos(struct acs *acs);
 void free_acs_infos(struct acs *acs);
-int acs_get_wingspan(struct acs *acs, char *icao);
+//int acs_get_wingspan(struct acs *acs, char *icao);
+void ac_fill_wingspan(struct acs* acs, struct ac* ac);
+
 struct ac *ac_under(struct acs *acs, struct subwin *sw);
 void ac_track(struct acs *acs, char *id, int rotnr);
 
+//openskynetwork
+int acs_osn_init(struct acs* acs);
+void acs_osn_free(struct acs* acs);
+void acs_osn_http_timer(void *arg);
+void acs_osn_downloaded_callback(struct zhttp *http);
+void acs_osn_load_from_main_thread(struct acs* acs, const char* data);
+#ifdef Z_HAVE_SDL
+void acs_osn_plot_info(struct subwin* sw, SDL_Rect* area, struct acs* acs);
+#endif
+
+//file
+int acs_file_init(struct acs* acs);
+void acs_file_free(struct acs* acs);
+void acs_file_read_feed(struct acs* acs);
+
+//adsb.one
+int acs_adsbone_init(struct acs* acs);
+void acs_adsbone_free(struct acs* acs);
+void acs_adsbone_http_timer(void* arg);
+void acs_adsbone_downloaded_callback(struct zhttp* http);
+void acs_adsbone_load_from_main_thread(struct acs* acs, const char* data);
+#ifdef Z_HAVE_SDL
+void acs_adsbone_plot_info(struct subwin* sw, SDL_Rect* area, struct acs* acs);
+#endif
+
 #endif
