@@ -276,6 +276,10 @@ typedef long long vint;
 #include <ftdi.h>
 #endif
 
+#ifdef HAVE_HIDAPI
+#include <hidapi.h>
+#endif
+
 /* zlib.h */
 #include <zglib.h>
 #include <zifaces.h>
@@ -284,6 +288,7 @@ typedef long long vint;
 #include <zsun.h>
 
 #define VERSION_STRING          VERSION
+#define AUTOSAVE 
 
 #define DEBUG
 
@@ -376,7 +381,7 @@ typedef long long vint;
 
 #define QSONR_WIDTH 20
 #define QSONR_HEIGHT 7
-#define BAND_WIDTH 16
+#define BAND_WIDTH 19
 #define ORIG_Y 1
 
 
@@ -597,4 +602,5 @@ struct takeoff{
 #pragma clang diagnostic ignored "-Winvalid-source-encoding"
 #endif
 
+#define OCT8TOR
 #endif /* __HEADER_H */     
