@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2022  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2023  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
     This program is free software; you can redistribute it and/or
@@ -297,6 +297,9 @@ static void really_close_ctest(void)
 	rain_reload();
 #endif   
 	chart_reload();
+#ifdef OCT8TOR
+	oct8tor_send_band(goct8tor, NULL);
+#endif
 }
 
 void menu_save_all(void *arg){
@@ -864,49 +867,47 @@ void subwins_menu(void *arg)
         return;
     }     */
 
-    if (!(mi = new_menu(3))) return;
-    add_to_menu(&mi, g_strdup(VTEXT(T_ADD_SUBWIN)), "", VTEXT(T_HK_ADD_SUBWIN), MENU_FUNC menu_add_subwin, GINT_TO_POINTER(0), 0);
-    add_to_menu(&mi, g_strdup(VTEXT(T_CLOSE_SUBWIN)), "", VTEXT(T_HK_CLOSE_SUBWIN), MENU_FUNC menu_close_subwin, NULL, 0);
-    add_to_menu(&mi, g_strdup(VTEXT(T_CHANGE_TYPE)), "", VTEXT(T_HK_CHANGE_TYPE), MENU_FUNC menu_add_subwin, GINT_TO_POINTER(1), 0);	  // TODO text
-    /*add_to_menu(&mi, g_strdup("Split window"), "", "S", MENU_FUNC menu_split_subwin, NULL, 0);
-    add_to_menu(&mi, g_strdup("Remove split"), "", "R", MENU_FUNC menu_remove_split, NULL, 0);*/
+    if (!(mi = new_menu(0x02 | 0x04))) return;
+    add_to_menu(&mi, g_strdup(VTEXT(T_ADD_SUBWIN)), g_strdup(""), VTEXT(T_HK_ADD_SUBWIN), MENU_FUNC menu_add_subwin, GINT_TO_POINTER(0), 0);
+    add_to_menu(&mi, g_strdup(VTEXT(T_CLOSE_SUBWIN)), g_strdup(""), VTEXT(T_HK_CLOSE_SUBWIN), MENU_FUNC menu_close_subwin, NULL, 0);
+    add_to_menu(&mi, g_strdup(VTEXT(T_CHANGE_TYPE)), g_strdup(""), VTEXT(T_HK_CHANGE_TYPE), MENU_FUNC menu_add_subwin, GINT_TO_POINTER(1), 0);	  // TODO text
 
 
-    if (arg == NULL) add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
+    if (arg == NULL) add_to_menu(&mi, g_strdup(""), g_strdup(""), M_BAR, NULL, NULL, 0);
     for (i = 0; i < gses->subwins->len; i++){
         struct subwin *sw = (struct subwin *)g_ptr_array_index(gses->subwins, i);
         if (sw != gses->ontop) continue;
 
         switch (sw->type){
             case SWT_KST:
-			    if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_OPEN_CONNECTION)), "", VTEXT(T_HK_OPEN_CONNECTION), kst_open_connection, sw, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_KILL_CONNECTION)), "", VTEXT(T_HK_KILL_CONNECTION), menu_kill_connection, sw, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_EXPORT_TO_TEXT_FILE)), "", VTEXT(T_HK_EXPORT_TO_TEXT_FILE), kst_export_text, sw, 0);
+			    if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), g_strdup(""), M_BAR, NULL, NULL, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_OPEN_CONNECTION)), g_strdup(""), VTEXT(T_HK_OPEN_CONNECTION), kst_open_connection, sw, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_KILL_CONNECTION)), g_strdup(""), VTEXT(T_HK_KILL_CONNECTION), menu_kill_connection, sw, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_EXPORT_TO_TEXT_FILE)), g_strdup(""), VTEXT(T_HK_EXPORT_TO_TEXT_FILE), kst_export_text, sw, 0);
 				//add_to_menu(&mi, g_strdup("OK1MZM: >>>> Press HERE <<<<"), "", "D", kst_dump_skip, sw, 0);
                 sep = 1;
                 break;
             case SWT_DXC:
-			    if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_OPEN_CONNECTION)), "", VTEXT(T_HK_OPEN_CONNECTION), dxc_open_connection, sw, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_KILL_CONNECTION)), "", VTEXT(T_HK_KILL_CONNECTION), menu_kill_connection, sw, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_EXPORT_TO_TEXT_FILE)), "", CTEXT(T_HK_EXPORT_TO_TEXT_FILE), dxc_export_text, sw, 0);
+			    if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), g_strdup(""), M_BAR, NULL, NULL, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_OPEN_CONNECTION)), g_strdup(""), VTEXT(T_HK_OPEN_CONNECTION), dxc_open_connection, sw, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_KILL_CONNECTION)), g_strdup(""), VTEXT(T_HK_KILL_CONNECTION), menu_kill_connection, sw, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_EXPORT_TO_TEXT_FILE)), g_strdup(""), CTEXT(T_HK_EXPORT_TO_TEXT_FILE), dxc_export_text, sw, 0);
                 sep = 1;
                 break;
             case SWT_SHELL:
-			    if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_KILL_CONNECTION)), "", VTEXT(T_HK_KILL_CONNECTION), menu_send_ctrlc, sw, 0);
+			    if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), g_strdup(""), M_BAR, NULL, NULL, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_KILL_CONNECTION)), g_strdup(""), VTEXT(T_HK_KILL_CONNECTION), menu_send_ctrlc, sw, 0);
                 sep = 1;
                 break;
 			case SWT_CHART:
-				if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_ADD_CONTEST)), "", VTEXT(T_HK_ADD_CONTEST), menu_chart_add_contest, sw, 0);
-				add_to_menu(&mi, g_strdup(VTEXT(T_ADD_FILES)), "", VTEXT(T_HK_ADD_FILES), menu_chart_add_files, sw, 0);
+				if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), g_strdup(""), M_BAR, NULL, NULL, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_ADD_CONTEST)), g_strdup(""), VTEXT(T_HK_ADD_CONTEST), menu_chart_add_contest, sw, 0);
+				add_to_menu(&mi, g_strdup(VTEXT(T_ADD_FILES)), g_strdup(""), VTEXT(T_HK_ADD_FILES), menu_chart_add_files, sw, 0);
                 sep = 1;
 				break;
 			case SWT_QRV:
-				if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
-                add_to_menu(&mi, g_strdup(VTEXT(T_ADD_CONTESTS)), "", VTEXT(T_HK_ADD_CONTESTS), menu_qrv_add_contest, sw, 0);
+				if (arg != NULL && sep == 0) add_to_menu(&mi, g_strdup(""), g_strdup(""), M_BAR, NULL, NULL, 0);
+                add_to_menu(&mi, g_strdup(VTEXT(T_ADD_CONTESTS)), g_strdup(""), VTEXT(T_HK_ADD_CONTESTS), menu_qrv_add_contest, sw, 0);
 				sep = 1;
 				break;
             default:
@@ -915,18 +916,20 @@ void subwins_menu(void *arg)
     }
 
 	if (arg == NULL) {
-	    if (sep) add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
+	    if (sep) add_to_menu(&mi, g_strdup(""), g_strdup(""), M_BAR, NULL, NULL, 0);
 
 		for (i = 0; i < gses->subwins->len; i++){
 			struct subwin *sw;
-			char *rtext = "";
+			char *rtext;
 
 			sw = (struct subwin *)g_ptr_array_index(gses->subwins,i);
 
-			if (i <= 9)
-				rtext = g_strdup_printf(VTEXT(T_ALTC), '0' + ((i + 1) % 10));
-			else if (i <= 19 && zsdl_get())
-				rtext = g_strdup_printf(VTEXT(T_CTRLC), '0' + ((i + 1) % 10));
+            if (i <= 9)
+                rtext = g_strdup_printf(VTEXT(T_ALTC), '0' + ((i + 1) % 10));
+            else if (i <= 19 && zsdl_get())
+                rtext = g_strdup_printf(VTEXT(T_CTRLC), '0' + ((i + 1) % 10));
+            else
+                rtext = g_strdup("");
 
 
 			sprintf(s,"%d", i);
@@ -1017,6 +1020,10 @@ void charset_sel_list(void *arg)
 static void terminal_options_ok(void *p)
 {
     redraw_terminal_cls();
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 static char *td_labels[] = { CTEXT(T_NO_FRAMES), CTEXT(T_VT_100_FRAMES), CTEXT(T_LINUX_OR_OS2_FRAMES), CTEXT(T_KOI8R_FRAMES), CTEXT(T_USE_11M), CTEXT(T_RESTRICT_FRAMES_IN_CP850_852), CTEXT(T_BLOCK_CURSOR), CTEXT(T_COLOR), CTEXT(T_UTF_8_IO), NULL };
@@ -1097,6 +1104,10 @@ void menu_set_language(void *arg)
 {
     set_language(GPOINTER_TO_INT(arg));
     redraw_terminal_cls();
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 void menu_language_list(void *arg)
@@ -1285,7 +1296,7 @@ static struct menu_item edit_menu[] = {
     {CTEXT(T_CW_BAND), "", CTEXT(T_HK_CW_BAND),  MENU_FUNC menu_cw_update_band, NULL, 0, 0},
     {CTEXT(T_EXC_CTEST), "", CTEXT(T_HK_EXC_CTEST),  MENU_FUNC menu_exc_update_contest, NULL, 0, 0},
     {CTEXT(T_EXC_BAND), "", CTEXT(T_HK_EXC_BAND),  MENU_FUNC menu_exc_update_band, NULL, 0, 0},
-    {CTEXT(T_IMPORT_EBW), "", CTEXT(T_HK_IMPORT_EBW),  MENU_FUNC menu_import_ebw, NULL, 0, 0},
+    {CTEXT(T_IMPORT_MMM), "", CTEXT(T_HK_IMPORT_MMM),  MENU_FUNC menu_import_mmm, NULL, 0, 0},
     {NULL, NULL, NULL, NULL, NULL, 0, 0}
 };
 
@@ -1303,7 +1314,7 @@ static struct menu_item edit_menu2[] = {
 #endif
 #endif
     {"", "", M_BAR, NULL, NULL, 0, 0},
-    {CTEXT(T_IMPORT_EBW), "", CTEXT(T_HK_IMPORT_EBW),  MENU_FUNC menu_import_ebw, NULL, 0, 0},
+    {CTEXT(T_IMPORT_MMM), "", CTEXT(T_HK_IMPORT_MMM),  MENU_FUNC menu_import_mmm, NULL, 0, 0},
     {NULL, NULL, NULL, NULL, NULL, 0, 0}
 };
 
@@ -1331,8 +1342,10 @@ void menu_save_rc(void *arg){
         c = g_strdup_printf(VTEXT(T_CANT_WRITE_S), filename);
         errbox(c, ret);
         g_free(c);
+//#ifndef AUTOSAVE
     }else{
         log_addf(VTEXT(T_SAVED_S), filename);
+//#endif
     }
 
     g_free(filename);
@@ -1571,6 +1584,12 @@ void select_spy3(void *itarg, void *menuarg){
 		rwbands = z_tokenize(peer, &tok2);
         if (!rwbands) continue;
 
+		if (menui == no){
+			spypeer_add(id, '*', 1);
+			return;
+		}
+		menui++;
+
 		for (c = rwbands; *c != '\0'; c++){
             if (menui == no){
                 spypeer_add(id, *c, 1);
@@ -1604,6 +1623,12 @@ void spy3(void *xxx){
 		rwbands = z_tokenize(peer, &tok2);
         if (!rwbands) continue;
 
+		if (!mi) mi = new_menu(3);
+		s = g_strdup_printf("%-21s %7s %s", id, "*", op);
+		if (strlen(s) > max) max = strlen(s);
+		add_to_menu(&mi, s, "", "", select_spy3, GINT_TO_POINTER(menui), 0);
+		menui++;
+
 		for (c = rwbands; *c != '\0'; c++){
 			struct band *b = find_band_by_bandchar(*c);
 			if (!b) continue;
@@ -1629,7 +1654,7 @@ void menu_discover_peers3(void *itarg, void *menuarg){
 	gnet->peers3 = g_string_sized_new(256);
 	progress(VTEXT(T_NETWORK_SCAN));
 	rel_write_all("D3 \n");
-	gnet->timer3_id = zselect_timer_new(zsel, 1000, (void (*)(void *))itarg, NULL);
+	gnet->timer3_id = zselect_timer_new(zsel, 2000, (void (*)(void *))itarg, NULL);
 }
 
 
@@ -1697,12 +1722,13 @@ static struct menu_item setup_menu[] = {
     {CTEXT(T_RIGOPTS), ">", CTEXT(T_HK_RIGOPTS), MENU_FUNC menu_rigs, NULL, 1, 0},
 #endif
     {CTEXT(T_ROTAROPTS), ">", CTEXT(T_HK_ROTAROPTS), MENU_FUNC menu_rotars, NULL, 1, 0},
-    {CTEXT(T_AUTOSAVE), "", CTEXT(T_HK_AUTOSAVE), MENU_FUNC menu_autosave, NULL, 0, 0},
     {CTEXT(T_NETWORK), "", CTEXT(T_HK_NETWORK), MENU_FUNC menu_network, NULL, 0, 0},
     {CTEXT(T_MISCOPTS), "", CTEXT(T_HK_MISCOPTS), MENU_FUNC misc_opts, NULL, 0, 0},
-    {CTEXT(T_TERMINAL_OPTIONS), "", CTEXT(T_HK_TERMINAL_OPTIONS), MENU_FUNC terminal_options, NULL, 0, 0},
+    //{CTEXT(T_TERMINAL_OPTIONS), "", CTEXT(T_HK_TERMINAL_OPTIONS), MENU_FUNC terminal_options, NULL, 0, 0},
     {CTEXT(T_HTTPD_OPTIONS)/*CTEXT(T_TERMINAL_OPTIONS)*/, "", CTEXT(T_HK_HTTP_OPTIONS)/*CTEXT(T_HK_TERMINAL_OPTIONS)*/, MENU_FUNC menu_httpd_opts, NULL, 0, 0},
+#ifndef Z_ANDROID
     {CTEXT(T_AC_OPTIONS)/*CTEXT(T_TERMINAL_OPTIONS)*/, "", CTEXT(T_HK_AC_OPTIONS)/*CTEXT(T_HK_TERMINAL_OPTIONS)*/, MENU_FUNC menu_ac_opts, NULL, 0, 0},
+#endif
 #if defined(HAVE_ALSA) || defined (HAVE_PORTAUDIO)
     {CTEXT(T_SDR_OPTIONS)/*CTEXT(T_TERMINAL_OPTIONS)*/, "", CTEXT(T_HK_SDR_OPTIONS)/*CTEXT(T_HK_TERMINAL_OPTIONS)*/, MENU_FUNC menu_sdr_opts, NULL, 0, 0},
 #endif
@@ -1712,8 +1738,10 @@ static struct menu_item setup_menu[] = {
     {"", "", M_BAR, NULL, NULL, 0, 0},
     {CTEXT(T_LOAD_CFG_NET), "", CTEXT(T_HK_LOAD_CFG_NET), MENU_FUNC menu_load_from_peer, send_config_request, 0, 0},/* todo */
     {CTEXT(T_LOAD_CW_NET), "", CTEXT(T_HK_LOAD_CW_NET), MENU_FUNC menu_load_from_peer, send_cwdb_request, 0, 0},/* todo */
+#ifndef AUTOSAVE
     {"", "", M_BAR, NULL, NULL, 0, 0},
     {CTEXT(T_SAVE_CFG), "", CTEXT(T_HK_SAVE_CFG), MENU_FUNC menu_save_rc, NULL, 0, 0},
+#endif
     {NULL, NULL, 0, NULL, NULL, 0, 0}
 };
 
