@@ -52,6 +52,10 @@ void refresh_cq_cw(void *xxx){
 	cq->cw_breakable = cw_breakable;
     cq->cw_ts  = atoi(cw_ts_str);
     cq->cw_allowifundef = cw_allowifundef;
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 char *cq_cw_msg[] = {
@@ -249,6 +253,10 @@ void refresh_cq_ssb(void *xxx){
 	cq->ssb_repeat = ssb_repeat;
 	cq->ssb_breakable = ssb_breakable;
     cq->ssb_ts  = atoi(ssb_ts_str);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 char *cq_ssb_msg[] = {
@@ -409,7 +417,7 @@ void menu_cq_ssb(void *arg){
 
 /******************* CWDAEMON **********************************/
 
-int cwda_type;
+int cwda_enabled, cwda_type;
 char cwda_device[MAX_STR_LEN];
 char cwda_hostname[MAX_STR_LEN];
 char udp_port_str[EQSO_LEN];
@@ -424,6 +432,7 @@ char autgive_str[EQSO_LEN];
 void refresh_cwda(void *xxx){
     
 	cfg->cwda_type=cwda_type;
+    if (!cwda_enabled) cfg->cwda_type = CWD_NONE;
     STORE_STR(cfg, cwda_device);
     STORE_STR(cfg, cwda_hostname);
     cfg->cwda_udp_port = atoi(udp_port_str);
@@ -441,6 +450,9 @@ void refresh_cwda(void *xxx){
 	free_cwdaemon(cwda);
 	cwda=init_cwdaemon();
 	progress(NULL);
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 
@@ -449,6 +461,7 @@ void menu_cwda(void *arg){
     struct dialog *d;
     int i;
     
+    cwda_enabled = cfg->cwda_type > 0;
 	cwda_type=cfg->cwda_type;
     safe_strncpy0(cwda_device, cfg->cwda_device, MAX_STR_LEN);
     safe_strncpy0(cwda_hostname, cfg->cwda_hostname, MAX_STR_LEN);
@@ -472,13 +485,21 @@ void menu_cwda(void *arg){
     d->refresh = (void (*)(void *))refresh_cwda;
     d->refresh_data = NULL;
     d->y0 = 1;
-    
+  
     d->items[i=0].type = D_CHECKBOX;
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char*)&cwda_enabled;
+    d->items[i].msg = CTEXT(T_ENABLED);//CTEXT(T_RIG_QRG_R2T);
+    d->items[i].wrap = 1;
+
+/*    d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_NONE;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (char *)&cwda_type;
-    d->items[i].msg = CTEXT(T_ROT_NONE);
+    d->items[i].msg = CTEXT(T_ROT_NONE);*/
 	
     d->items[++i].type = D_CHECKBOX; 
     d->items[i].gid = 1;
@@ -502,13 +523,24 @@ void menu_cwda(void *arg){
 	d->items[i].msg = CTEXT(T_TTYSSINGLE);
 	d->items[i].wrap = 1;
 
+#ifdef Z_HAVE_LIBFTDI
 	d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_DAVAC4;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (char *)&cwda_type;
     d->items[i].msg = "Davac4";
-    
+#endif
+
+#ifdef HAVE_HIDAPI
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = CWD_DAVAC5;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char*)&cwda_type;
+    d->items[i].msg = "Davac5";
+#endif
+
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_CWD;
@@ -642,7 +674,16 @@ void menu_cwda(void *arg){
     d->items[i].gid = 0;
     d->items[i].fn = dlg_winkey_opts;
     d->items[i].text = VTEXT(T_WINKEYOPTS);
+    d->items[i].wrap = 1;
+    
+#ifdef HAVE_HIDAPI
+
+    d->items[++i].type = D_BUTTON;    
+    d->items[i].gid = 0;
+    d->items[i].fn = dlg_davac5_opts;
+    d->items[i].text = TRANSLATE("Davac5 options");//VTEXT(T_WINKEYOPTS);
     d->items[i].wrap = 2;
+#endif
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
@@ -673,8 +714,9 @@ extern char ssbd_format_str[MAX_STR_LEN];
 extern char ssbd_oss_src[MAX_STR_LEN];
 extern char ssbd_alsa_src[MAX_STR_LEN];
 #ifdef HAVE_PORTAUDIO
-char ssbd_pa_play_src[MAX_STR_LEN];
-char ssbd_pa_rec_src[MAX_STR_LEN];
+int ssbd_pa_play_hi, ssbd_pa_rec_hi;
+char ssbd_pa_play_s[MAX_STR_LEN];
+char ssbd_pa_rec_s[MAX_STR_LEN];
 #endif
 
 char *record_format_msg[]={
@@ -845,8 +887,8 @@ static void pa_play_src_func (void *arg){
     int active = GPOINTER_TO_INT(arg);
 
     const PaDeviceInfo *di = Pa_GetDeviceInfo(active);
-    safe_strncpy0(ssbd_pa_play_src, di->name, MAX_STR_LEN);
-    cfg->ssbd_pa_play = active;
+    ssbd_pa_play_hi = di->hostApi;
+    safe_strncpy0(ssbd_pa_play_s, di->name, MAX_STR_LEN);
     redraw_later();
 }
 
@@ -862,7 +904,7 @@ static int dlg_pa_play_src(struct dialog_data *dlg, struct dialog_item_data *di)
     	const PaDeviceInfo *di = Pa_GetDeviceInfo(i);
 		if (di->maxOutputChannels <= 0) continue;
         add_to_menu(&mi, g_strdup_printf("%d: %s", i, di->name), "", "", MENU_FUNC pa_play_src_func, GINT_TO_POINTER(i), 0);
-		if (i == cfg->ssbd_pa_play) sel = n;
+		if (ssbd_pa_play_hi == di->hostApi && strcmp(di->name, ssbd_pa_play_s) == 0) sel = n;
         n++;
     }
     do_menu_selected(mi, NULL, sel);
@@ -875,8 +917,8 @@ static void pa_rec_src_func(void *arg){
     int active = GPOINTER_TO_INT(arg);
 
     const PaDeviceInfo *di = Pa_GetDeviceInfo(active);
-    safe_strncpy0(ssbd_pa_rec_src, di->name, MAX_STR_LEN);
-    cfg->ssbd_pa_rec = active;
+    ssbd_pa_rec_hi = di->hostApi;
+    safe_strncpy0(ssbd_pa_rec_s, di->name, MAX_STR_LEN);
     redraw_later();
 }
 
@@ -892,7 +934,7 @@ static int dlg_pa_rec_src(struct dialog_data *dlg, struct dialog_item_data *di){
     	const PaDeviceInfo *di = Pa_GetDeviceInfo(i);
 		if (di->maxInputChannels <= 0) continue;
         add_to_menu(&mi, g_strdup_printf("%d: %s", i, di->name), "", "", MENU_FUNC pa_rec_src_func, GINT_TO_POINTER(i), 0);
-		if (i == cfg->ssbd_pa_rec) sel = n;
+        if (ssbd_pa_rec_hi == di->hostApi && strcmp(di->name, ssbd_pa_rec_s) == 0) sel = n;
         n++;
     }
     do_menu_selected(mi, NULL, sel);
@@ -962,6 +1004,14 @@ void refresh_ssbd(void *xxx){
     STORE_STR(cfg, ssbd_oss_src);
     dbg("refresh_ssbd: ssbd_oss_src='%s'\n", cfg->ssbd_oss_src);
 
+#ifdef HAVE_PORTAUDIO
+    STORE_INT(cfg, ssbd_pa_play_hi);
+    STORE_INT(cfg, ssbd_pa_rec_hi);
+    STORE_STR(cfg, ssbd_pa_play_s);
+    STORE_STR(cfg, ssbd_pa_rec_s);
+
+#endif
+
     STORE_STR(cfg, ssbd_hostname);
     STORE_SINT(cfg, ssbd_udp_port);
 
@@ -969,6 +1019,10 @@ void refresh_ssbd(void *xxx){
     sw_scope_clear_scopes();
     gssbd=init_ssbd();
 	progress(NULL);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 void do_ssbd(void){
@@ -1040,21 +1094,31 @@ void do_ssbd(void){
     }
 
 #ifdef HAVE_PORTAUDIO
+    if (cfg->ssbd_pa_play_s != NULL && strlen(cfg->ssbd_pa_play_s) >= 0) {
+        cfg->ssbd_pa_play = pa_find_device(cfg->ssbd_pa_play_s, 1, 0);
+    }
 	if (cfg->ssbd_pa_play < 0) cfg->ssbd_pa_play = Pa_GetDefaultOutputDevice();
     di = Pa_GetDeviceInfo(cfg->ssbd_pa_play);
 	if (di){
-		safe_strncpy(ssbd_pa_play_src, di->name, MAX_STR_LEN);
+        ssbd_pa_play_hi = di->hostApi;
+        safe_strncpy(ssbd_pa_play_s, di->name, MAX_STR_LEN);
 	}else{
-		safe_strncpy0(ssbd_pa_play_src, VTEXT(T_RECSRC_NONE), MAX_STR_LEN);
+        ssbd_pa_play_hi = -1;
+        safe_strncpy0(ssbd_pa_play_s, VTEXT(T_RECSRC_NONE), MAX_STR_LEN);
 	}
     
-	if (cfg->ssbd_pa_rec < 0) cfg->ssbd_pa_rec = Pa_GetDefaultInputDevice();
+    if (cfg->ssbd_pa_rec_s != NULL && strlen(cfg->ssbd_pa_rec_s) >= 0) {
+        cfg->ssbd_pa_rec = pa_find_device(cfg->ssbd_pa_rec_s, 1, 0);
+    }
+    if (cfg->ssbd_pa_rec < 0) cfg->ssbd_pa_rec = Pa_GetDefaultInputDevice();
     di = Pa_GetDeviceInfo(cfg->ssbd_pa_rec);
 	if (di){
-		safe_strncpy(ssbd_pa_rec_src, di->name, MAX_STR_LEN);
+        ssbd_pa_rec_hi = di->hostApi;
+        safe_strncpy(ssbd_pa_rec_s, di->name, MAX_STR_LEN);
 	}else{
-		safe_strncpy0(ssbd_pa_rec_src, VTEXT(T_RECSRC_NONE), MAX_STR_LEN);
-	}
+        ssbd_pa_rec_hi = -1;
+        safe_strncpy0(ssbd_pa_rec_s, VTEXT(T_RECSRC_NONE), MAX_STR_LEN);
+    }
 #endif
     
 
@@ -1282,14 +1346,14 @@ void do_ssbd(void){
     d->items[++i].type = D_BUTTON;  
     d->items[i].gid  = 0;
     d->items[i].fn   = dlg_pa_play_src;
-    d->items[i].text = ssbd_pa_play_src;
+    d->items[i].text = ssbd_pa_play_s;
     d->items[i].msg = CTEXT(T_PA_PLAY);
     d->items[i].wrap = 1;
     
     d->items[++i].type = D_BUTTON;  
     d->items[i].gid  = 0;
     d->items[i].fn   = dlg_pa_rec_src;
-    d->items[i].text = ssbd_pa_rec_src;
+    d->items[i].text = ssbd_pa_rec_s;
     d->items[i].msg = CTEXT(T_PA_REC);
     d->items[i].wrap = 2;
 #endif
@@ -1362,6 +1426,10 @@ void refresh_net(void *xxx){
 	if (raise){
 		sw_raise_or_new(SWT_LOG);
 	}
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 
