@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2021  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2023  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -45,10 +45,19 @@ void refresh_misc_opts(void *xxx)
 {
     struct config_band *confb;
     int gfx_x_ch, gfx_y_ch, gfx_x_px, gfx_y_px;
-    int new_w, new_h, resize, new_fh;
+    int new_w = 800, new_h = 600, resize, new_fh;
+
 
 	resize=0;
-	if (cfg->loglines != atoi(loglines_str)) resize++;
+    if (cfg->loglines != atoi(loglines_str)) {
+        resize++;
+#ifdef Z_HAVE_SDL
+        if (cfg->loglines != atoi(loglines_str)) {
+            new_w = sdl->screen->w;
+            new_h = sdl->screen->h;
+        }
+#endif
+    }
 
     STORE_SINT(cfg,loglines);
     STORE_SINT(cfg,skedcount);
@@ -79,9 +88,6 @@ void refresh_misc_opts(void *xxx)
 	STORE_INT(cfg, adif_export_mode_t2r);
 	STORE_STR(cfg, logdirpath);
     
-    new_w=800;
-    new_h=600;
-
     new_fh = atoi(fontheight_str);
 
 #ifdef Z_HAVE_SDL    
@@ -118,7 +124,7 @@ void refresh_misc_opts(void *xxx)
 			resize++;
 		}
 
-        if (resize && new_w <= 2048 && new_h <= 2048) {
+		if (resize && new_w <= 3840 && new_h <= 3840) {
             cfg->gfx_x = new_w;
             cfg->gfx_y = new_h;
             sdl_setvideomode(new_w, new_h, 0);
@@ -133,6 +139,10 @@ void refresh_misc_opts(void *xxx)
 
 	set_logs_dir();
 
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 void set_adif_export_mode(void *arg, void *arg2){
@@ -210,7 +220,8 @@ void misc_opts(void *arg)
 #ifdef Z_HAVE_SDL
 	if (sdl){
 		d->items[++i].type = D_FIELD;
-		d->items[i].gid  = 13;
+		d->items[i].fn = check_number;
+		d->items[i].gid = 13;
 		d->items[i].gnum = 32;
 		d->items[i].maxl = 5;
 		d->items[i].dlen = EQSO_LEN;
@@ -448,7 +459,7 @@ void menu_fixqrg(void *itdata, void *menudata){
 
 /************************** winkey ******************************/
 
-int wk_wk2, wk_usepot, wk_usebut, wk_keymode, wk_swap;
+int wk_wk2, wk_usepot, wk_usebut, wk_keymode, wk_swap, wk_echoback;
 
 
 void refresh_winkey_opts(void *xxx)
@@ -458,6 +469,7 @@ void refresh_winkey_opts(void *xxx)
     STORE_INT(cfg, wk_usebut);
     STORE_INT(cfg, wk_keymode);
     STORE_INT(cfg, wk_swap);
+    STORE_INT(cfg, wk_echoback);
 }
 
 char *winkey_opts_msg[] = {
@@ -468,7 +480,8 @@ char *winkey_opts_msg[] = {
     CTEXT(T_WK_KM_A), 
     CTEXT(T_WK_KM_U), 
     CTEXT(T_WK_KM_G), 
-    CTEXT(T_WK_SWAP), 
+    CTEXT(T_WK_SWAP),
+    CTEXT(T_WK_ECHOBACK),
     "",
     ""
 };
@@ -488,9 +501,11 @@ void winkey_opts_fn(struct dialog_data *dlg)
     min_group_width(term, winkey_opts_msg + 3, dlg->items + 3, 4, &min);
     max_group_width(term, winkey_opts_msg + 7, dlg->items + 1, 1, &max);
     min_group_width(term, winkey_opts_msg + 7, dlg->items + 1, 1, &min);
+    max_group_width(term, winkey_opts_msg + 8, dlg->items + 1, 1, &max);
+    min_group_width(term, winkey_opts_msg + 8, dlg->items + 1, 1, &min);
     
-    max_buttons_width(term, dlg->items + 8, 2, &max);
-    min_buttons_width(term, dlg->items + 8, 2, &min);
+    max_buttons_width(term, dlg->items + 9, 2, &max);
+    min_buttons_width(term, dlg->items + 9, 2, &min);
     
     w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
     if (w > max) w = max;
@@ -504,8 +519,9 @@ void winkey_opts_fn(struct dialog_data *dlg)
     dlg_format_group(NULL, term, winkey_opts_msg + 1, dlg->items + 1, 2, 0, &y, w, &rw);
     dlg_format_group(NULL, term, winkey_opts_msg + 3, dlg->items + 3, 4, 0, &y, w, &rw);
     dlg_format_group(NULL, term, winkey_opts_msg + 7, dlg->items + 7, 1, 0, &y, w, &rw);
+    dlg_format_group(NULL, term, winkey_opts_msg + 8, dlg->items + 8, 1, 0, &y, w, &rw);
     y++;
-    dlg_format_buttons(NULL, term, dlg->items +8, 2, 0, &y, w, &rw, AL_LEFT);
+    dlg_format_buttons(NULL, term, dlg->items +9, 2, 0, &y, w, &rw, AL_LEFT);
     
     
     w = rw;
@@ -521,8 +537,9 @@ void winkey_opts_fn(struct dialog_data *dlg)
     dlg_format_group(term, term, winkey_opts_msg + 1, dlg->items + 1, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group(term, term, winkey_opts_msg + 3, dlg->items + 3, 4, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group(term, term, winkey_opts_msg + 7, dlg->items + 7, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    dlg_format_group(term, term, winkey_opts_msg + 8, dlg->items + 8, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     y++;
-    dlg_format_buttons(term, term, dlg->items +8, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+    dlg_format_buttons(term, term, dlg->items +9, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
 }
 
 void winkey_opts(void *arg)
@@ -535,6 +552,7 @@ void winkey_opts(void *arg)
     wk_usebut = cfg->wk_usebut;
     wk_keymode = cfg->wk_keymode;
     wk_swap = cfg->wk_swap;
+    wk_echoback = cfg->wk_echoback;
 
 
     if (!(d = (struct dialog *)g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
@@ -591,17 +609,22 @@ void winkey_opts(void *arg)
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (char *)&wk_swap;
 
-    
-    d->items[++i].type = D_BUTTON;   /* 8 */
+    d->items[++i].type = D_CHECKBOX;       /* 8 */
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&wk_echoback;
+
+    d->items[++i].type = D_BUTTON;   /* 9 */
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
     d->items[i].text = VTEXT(T_OK);
-    
+
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
     d->items[i].text = VTEXT(T_CANCEL);
-    
+
     d->items[++i].type = D_END;
     do_dialog(d, getml(d, NULL));
 }
@@ -629,6 +652,7 @@ char rig_lo_str[MAX_STR_LEN];
 char rig_poll_ms_str[MAX_STR_LEN];
 int rig_qrg_r2t, rig_qrg_t2r, rig_mode_t2r, rig_clr_rit, rig_ptt_t2r, rig_verbose;
 char rig_ptt_t2r_str[EQSO_LEN];
+char rig_conf[MAX_STR_LEN];
 
 int new_model = 0;
 char new_model_name[EQSO_LEN];
@@ -754,10 +778,15 @@ void refresh_rig_opts(void *xxx){
     STORE_INT(crig, rig_clr_rit);
     STORE_INT(crig, rig_ptt_t2r);
     STORE_INT(crig, rig_verbose);
+    STORE_STR(crig, rig_conf);
 
     free_trigs(gtrigs);
     init_trigs();        
 	progress(NULL);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 void rig_opts_fn(struct dialog_data *dlg)
@@ -799,7 +828,7 @@ void menu_rig_opts(void *arg)
         rig_clr_rit = 0;       
 		rig_ptt_t2r = 0;
         rig_verbose = 0;        
-        
+        strcpy(rig_conf, "");
     }else{
         crig = (struct config_rig *)g_ptr_array_index(cfg->crigs, rigi);
         LOAD_INT(crig, rig_enabled);
@@ -811,6 +840,8 @@ void menu_rig_opts(void *arg)
 		LOAD_SHEX2(crig, rig_civaddr);
         LOAD_SINT(crig, rig_ssbcw_shift);
         LOAD_SDBL(crig, rig_lo);
+        if (crig->rig_poll_ms < 100) crig->rig_poll_ms = 300;
+        if (crig->rig_poll_ms > 9999) crig->rig_poll_ms = 9999;
         LOAD_SINT(crig, rig_poll_ms);
         LOAD_INT(crig, rig_qrg_r2t);
         LOAD_INT(crig, rig_qrg_t2r);
@@ -818,6 +849,7 @@ void menu_rig_opts(void *arg)
         LOAD_INT(crig, rig_clr_rit);
         LOAD_INT(crig, rig_ptt_t2r);
         LOAD_INT(crig, rig_verbose);
+        LOAD_STR(crig, rig_conf);
     }
 
 
@@ -873,7 +905,14 @@ void menu_rig_opts(void *arg)
 	d->items[i].data = (char *)GINT_TO_POINTER(rig_model);
     d->items[i].text = VTEXT(T_RIG_CHOOSE);
     d->items[i].wrap = 1;
-    
+
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = MAX_STR_LEN;
+    d->items[i].data = rig_conf;
+    d->items[i].maxl = 35;
+    d->items[i].msg = TRANSLATE("Config:");//CTEXT(T_RIG_DEVICE);
+    d->items[i].wrap = 1;
+
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = rig_speed_str;
@@ -1008,7 +1047,13 @@ void menu_rig_delete(void *arg){
     g_ptr_array_remove_index(cfg->crigs, rigi);
 	
     free_trigs(gtrigs);
-    init_trigs();        
+    init_trigs();    
+
+    progress(NULL);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 
@@ -1129,9 +1174,15 @@ void refresh_rotar_opts(void *xxx){
     STORE_SINT(crot, rot_saddr);
     STORE_STR (crot, rot_rem_rotstr); 
 
+MUTEX_LOCK(rotars);
     free_rotars();
     init_rotars();
 	progress(NULL);
+MUTEX_UNLOCK(rotars);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
         
 
@@ -1176,6 +1227,8 @@ void menu_rotar_opts(void *arg){
         LOAD_SHEX4(crot, rot_pid);
         LOAD_STR  (crot, rot_serial); 
         LOAD_SINT (crot, rot_timeout_ms);
+        if (crot->rot_poll_ms < 300) crot->rot_poll_ms = 500;
+        if (crot->rot_poll_ms > 2000) crot->rot_poll_ms = 2000;
         LOAD_SINT (crot, rot_poll_ms);
         LOAD_SINT (crot, rot_beamwidth);
 		LOAD_SINT (crot, rot_offset);
@@ -1388,6 +1441,13 @@ void menu_rotar_delete(void *arg){
     free_config_rotar(crot);
     g_ptr_array_remove_index(cfg->crotars, roti);
 
+    free_rotars();
+    init_rotars();
+    progress(NULL);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 void menu_rotars_delete(void *arg)
@@ -1420,7 +1480,15 @@ void menu_rotars(void *arg)
     
     if (!(mi = new_menu(3))) return;
 
-    for (i=0; i<cfg->crotars->len; i++){
+#ifdef OCT8TOR
+	add_to_menu(&mi,
+		g_strdup(TRANSLATE("Oct8tor")),
+		TRANSLATE("O"),
+		TRANSLATE("O"),
+		MENU_FUNC menu_oct8tor_opts, GINT_TO_POINTER(-1), 0);
+#endif
+
+	for (i = 0; i<cfg->crotars->len; i++){
         struct config_rotar *rot = (struct config_rotar *)g_ptr_array_index(cfg->crotars, i);
         char *c;
 
@@ -1442,7 +1510,10 @@ void menu_rotars(void *arg)
             rotstrs[rot->nr],
             MENU_FUNC menu_rotar_opts, GINT_TO_POINTER(i), 0);
     }
-    add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
+
+	if (cfg->crotars->len > 0){
+		add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
+	}
     
     add_to_menu(&mi, 
         g_strdup(VTEXT(T_ROT_ADD)),
