@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2012-2018 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2012-2023 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -38,6 +38,10 @@ void refresh_httpd_opts(void *arg){
 	free_httpd(httpd);
 	httpd = init_httpd();
 	progress(NULL);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 void menu_httpd_opts(void *arg){
@@ -122,13 +126,14 @@ void menu_httpd_opts(void *arg){
 
 /* open contest from net */
 
-int load_cw, load_name, load_qrv;
+int load_def, load_cw, load_name, load_qrv;
 
 void refresh_load_from_net(void *arg){
 	int load_opts = 0;
     int tok = 0, menui = 0, no = GPOINTER_TO_INT(arg);
 	char *peers3, *peer, *id, *tname, *c;
 
+	if (load_def)  load_opts |= LO_DEF;
 	if (load_cw)   load_opts |= LO_CW;
 	if (load_name) load_opts |= LO_NAME;
 	if (load_qrv)  load_opts |= LO_QRV;
@@ -161,6 +166,7 @@ void load_from_net(void *arg){
 	char *peers3, *peer, *id;
 	int no = GPOINTER_TO_INT(arg);
 
+	load_def = 1;
 	load_cw = 1;
 	load_name = 1;
 	load_qrv = 1;
@@ -176,6 +182,7 @@ void load_from_net(void *arg){
 			z_strip_from(peerid, ':');
 			
 			if (strcmp(myid, peerid) == 0){
+				load_def = 0;
 				load_cw = 0;
 				load_name = 0;
 				load_qrv = 0;
@@ -200,16 +207,24 @@ void load_from_net(void *arg){
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (char *)&load_cw;
-    d->items[i].msg = CTEXT(T_LOAD_CWDB);//CTEXT(T_RIG_QRG_R2T);
+    d->items[i].data = (char *)&load_def;
+    d->items[i].msg = CTEXT(T_LOAD_DEF);
     d->items[i].wrap = 1;
 
-	d->items[++i].type = D_CHECKBOX;      
+	d->items[++i].type = D_CHECKBOX;  
+	d->items[i].gid = 0;
+	d->items[i].gnum = 1;
+	d->items[i].dlen = sizeof(int);
+	d->items[i].data = (char *)&load_cw;
+	d->items[i].msg = CTEXT(T_LOAD_CWDB);
+	d->items[i].wrap = 1;
+
+	d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 2;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (char *)&load_name;
-    d->items[i].msg = CTEXT(T_LOAD_NAMEDB);//CTEXT(T_RIG_QRG_R2T);
+    d->items[i].msg = CTEXT(T_LOAD_NAMEDB);
     d->items[i].wrap = 1;
 
 	d->items[++i].type = D_CHECKBOX;      
@@ -217,7 +232,7 @@ void load_from_net(void *arg){
     d->items[i].gnum = 2;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (char *)&load_qrv;
-    d->items[i].msg = CTEXT(T_LOAD_QRVDB);//CTEXT(T_RIG_QRG_R2T);
+    d->items[i].msg = CTEXT(T_LOAD_QRVDB);
     d->items[i].wrap = 2;
     
     d->items[++i].type = D_BUTTON;
@@ -340,27 +355,65 @@ void menu_set_mode(void *itdata, void *menudata)
 
 //---- AS --------------------------------------------------------------------
 
-int ac_enable;       
-char ac_url[MAX_STR_LEN];        
-char ac_kfactor_str[DBL_LEN];   
+
+int ac_enable, ac_provider;
+char ac_osn_username[MAX_STR_LEN], ac_osn_password[MAX_STR_LEN];
+char ac_osn_minutes_str[EQSO_LEN];
+char ac_filename[MAX_STR_LEN], new_ac_filename[MAX_STR_LEN];
+char ac_adsbone_minutes_str[EQSO_LEN];
+char ac_centerwwl[EQSO_LEN];
+char ac_kfactor_str[DBL_LEN];
 char ac_arwidth_str[DBL_LEN];   
 char ac_minelev_str[DBL_LEN];   
 char ac_maxelev_str[DBL_LEN];   
 char ac_mindur_str[DBL_LEN];       
 char ac_maxdelta_str[DBL_LEN];  
 char ac_minalt_str[DBL_LEN];
+int ac_imperial;
 char kst_maxqrb_str[MAX_STR_LEN];
 
+// AC filename
+int ac_filename_index = -1;
+
+void ac_opts_fn(struct dialog_data* dlg)
+{
+    if (strlen(new_ac_filename) > 0 && ac_filename_index >= 0) {
+        g_strlcpy(dlg->items[ac_filename_index].cdata, new_ac_filename, MAX_STR_LEN);
+        dlg->items[ac_filename_index].cpos = strlen(new_ac_filename);
+        strcpy(new_ac_filename, "");
+    }
+    input_field_fn_i(dlg, ac_filename_index);
+}
+
+void refresh_as_filename(void* xxx, char* filename) {
+    g_strlcpy(new_ac_filename, filename, MAX_STR_LEN);
+    redraw_later();
+}
+
+static int dlg_as_filename(struct dialog_data* dlg, struct dialog_item_data* di) {
+    zfiledlg_open(zfiledlg, zsel, refresh_as_filename, di, "", "json");
+    return 0;
+}
+
+
 void refresh_ac_opts(void *arg){
     STORE_INT(cfg, ac_enable);
-    STORE_STR(cfg, ac_url);
+    STORE_INT(cfg, ac_provider);
+    STORE_STR(cfg, ac_osn_username);
+    STORE_STR(cfg, ac_osn_password);
+    STORE_SINT(cfg, ac_osn_minutes);
+    STORE_SINT(cfg, ac_adsbone_minutes);
+    STORE_STR(cfg, ac_filename);
+    z_unix(cfg->ac_filename);
+    STORE_STR(cfg, ac_centerwwl);
     STORE_SDBL(cfg, ac_kfactor);
     STORE_SDBL(cfg, ac_arwidth);
     STORE_SDBL(cfg, ac_minelev);
     STORE_SDBL(cfg, ac_maxelev);
     STORE_SINT(cfg, ac_mindur);
     STORE_SDBL(cfg, ac_maxdelta);
-    STORE_SINT(cfg, ac_minalt);
+	STORE_SINT(cfg, ac_minalt);
+	STORE_INT(cfg, ac_imperial);
 	STORE_SINT(cfg, kst_maxqrb);
 
 
@@ -368,14 +421,26 @@ void refresh_ac_opts(void *arg){
 	gacs = init_acs();
 	resize_terminal(NULL);
 	progress(NULL);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
+
 void menu_ac_opts(void *arg){
     struct dialog *d;
     int i;
 
-    ac_enable = cfg->ac_enable;
-    g_snprintf(ac_url, MAX_STR_LEN, "%s", cfg->ac_url);
+    LOAD_INT(cfg, ac_enable);
+    LOAD_INT(cfg, ac_provider);
+    LOAD_STR(cfg, ac_osn_username);
+    LOAD_STR(cfg, ac_osn_password);
+    LOAD_SINT(cfg, ac_osn_minutes);
+    LOAD_SINT(cfg, ac_adsbone_minutes);
+    g_snprintf(ac_filename, MAX_STR_LEN, "%s", cfg->ac_filename);
+    z_wokna(ac_filename);
+    LOAD_STR(cfg, ac_centerwwl);
     g_snprintf(ac_kfactor_str, DBL_LEN, "%3.3f", cfg->ac_kfactor);
     g_snprintf(ac_arwidth_str, DBL_LEN, "%3.1f", cfg->ac_arwidth);
     g_snprintf(ac_minelev_str, DBL_LEN, "%3.1f", cfg->ac_minelev);
@@ -383,13 +448,15 @@ void menu_ac_opts(void *arg){
     g_snprintf(ac_mindur_str,  DBL_LEN, "%d", cfg->ac_mindur);
     g_snprintf(ac_maxdelta_str, DBL_LEN, "%3.1f", cfg->ac_maxdelta);
     g_snprintf(ac_minalt_str, DBL_LEN, "%d", cfg->ac_minalt);
+    LOAD_INT(cfg, ac_imperial);
 	LOAD_SINT(cfg, kst_maxqrb);
 
 
-    d = (struct dialog *)g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
-    memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
+    d = (struct dialog *)g_malloc(sizeof(struct dialog) + 30 * sizeof(struct dialog_item));
+    memset(d, 0, sizeof(struct dialog) + 30 * sizeof(struct dialog_item));
     d->title = VTEXT(T_AC_OPTIONS);
     d->fn = dlg_pf_fn;
+    d->fn2 = ac_opts_fn;
     d->refresh = (void (*)(void *))refresh_ac_opts;
     d->y0 = 1;
 
@@ -400,15 +467,84 @@ void menu_ac_opts(void *arg){
     d->items[i].data = (char *)&ac_enable;
     d->items[i].msg = CTEXT(T_ENABLE_AC);
     d->items[i].wrap = 1;
+    /////////////////////////////////////////////////////////////
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = AC_ADSBONE;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char*)&ac_provider;
+    d->items[i].msg = TRANSLATE("ADS-B One");
+
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = ac_adsbone_minutes_str;
+    d->items[i].maxl = 4;
+    d->items[i].gid = 1;
+    d->items[i].gnum = 60;
+    d->items[i].msg = TRANSLATE("Interval [minutes]");
+    d->items[i].wrap = 1;
+    /////////////////////////////////////////////////////////////
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = AC_FILE;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char*)&ac_provider;
+    d->items[i].msg = TRANSLATE("File (Firefox)");
 
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = ac_url;
+    d->items[i].data = ac_filename;
     d->items[i].maxl = 42;
-    d->items[i].msg  = CTEXT(T_AC_URL);
+    d->items[i].msg = TRANSLATE("Feed filename");//CTEXT(T_AC_URL);
+    d->items[i].tabcompl = 1;
+    ac_filename_index = i;
+#ifdef Z_HAVE_SDL
+    if (sdl) {
+        d->items[++i].type = D_BUTTON;
+        d->items[i].fn = dlg_as_filename;
+        d->items[i].text = CTEXT(T_RIG_CHOOSE);
+    }
+    d->items[i].wrap++;
+#endif
+    /////////////////////////////////////////////////////////////
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = AC_OSN;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char*)&ac_provider;
+    d->items[i].msg = TRANSLATE("OpenSkyNetwork");
+
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = MAX_STR_LEN;
+    d->items[i].data = ac_osn_username;
+    d->items[i].maxl = 10;
+    d->items[i].msg = TRANSLATE("Username");
+
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = MAX_STR_LEN;
+    d->items[i].data = ac_osn_password;
+    d->items[i].maxl = 10;
+    d->items[i].msg = TRANSLATE("Password");
+
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = ac_osn_minutes_str;
+    d->items[i].maxl = 4;
+    d->items[i].gid = 2;
+    d->items[i].gnum = 60;
+    d->items[i].msg = TRANSLATE("Interval [minutes]");
     d->items[i].wrap = 1;
 
-    d->items[++i].type = D_FIELD;      
+//////////////////////////////////////////////////////
+    d->items[i].wrap++;
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = ac_centerwwl;
+    d->items[i].maxl = 8;
+    d->items[i].msg = TRANSLATE("Alternative center WWL");
+    d->items[i].wrap = 1;
+
+    d->items[++i].type = D_FIELD;
     d->items[i].maxl = 7;
     d->items[i].dlen = DBL_LEN;
     d->items[i].data = (char *)&ac_kfactor_str;
@@ -453,14 +589,22 @@ void menu_ac_opts(void *arg){
     d->items[i].fn   = check_number;
     d->items[i].gid  = 0;
     d->items[i].gnum = 20000;
-    d->items[i].msg = TRANSLATE("Minimal AC altitude [m]");
+    d->items[i].msg = TRANSLATE("Minimal AS altitude [m]");
     d->items[i].wrap = 1;
 
+	d->items[++i].type = D_CHECKBOX;
+	d->items[i].gid = 0;
+	d->items[i].gnum = 1;
+	d->items[i].dlen = sizeof(int);
+	d->items[i].data = (char *)&ac_imperial;
+	d->items[i].msg = TRANSLATE("Display AS in imperial units");
+	d->items[i].wrap = 1;
+
 	d->items[++i].type = D_FIELD;
 	d->items[i].dlen = MAX_STR_LEN;
 	d->items[i].data = (char*)&kst_maxqrb_str;
 	d->items[i].maxl = 6;
-	d->items[i].msg = TRANSLATE("Maximal QRB for KST");// CTEXT(T_KST_MAXQRB);
+	d->items[i].msg = TRANSLATE("Maximal QRB for KST [km]");// CTEXT(T_KST_MAXQRB);
 	d->items[i].wrap = 2;
 
 
@@ -653,6 +797,10 @@ void refresh_sdr_opts(void *arg){
 
 
 	progress(NULL);
+
+#ifdef AUTOSAVE
+    menu_save_rc(NULL);
+#endif
 }
 
 int sdr_check_range(struct dialog_data *dlg, struct dialog_item_data *di){
