@@ -297,6 +297,19 @@ int pa_set_sdr_format(struct dsp *dsp, int frames, int speed, int rec){
 
 }    
 
+int pa_find_device(const char* name, int play, int rec) {
+	int n = Pa_GetDeviceCount();
+	int i;
+	for (i = 0; i < n; i++)
+	{
+		const PaDeviceInfo* di = Pa_GetDeviceInfo(i);
+		if (play && di->maxOutputChannels == 0) continue;
+		if (rec && di->maxInputChannels == 0) continue;
+		if (strcmp(di->name, name) == 0) return i;
+	}
+	return -1;
+}
+
 #endif
 
 
