@@ -154,7 +154,7 @@ struct qrv_item *qrv_add(gchar *call, gchar *wwl, int qrv_int, int wkd[32], cons
             zg_free0(qi->text);
             qi->text = g_strdup(text);
         }
-        qi->kst_time = kst_time;
+        qi->kst_time = Z_MAX(qi->kst_time, kst_time);
 		for (wi = 0; wi < 32; wi++) qi->wkd[wi] += wkd[wi];
         return qi;
     }
@@ -300,7 +300,7 @@ struct qrv_item *qrv_kst_add(char *call, char *wwl, char *fullname){
 		}
 	}
 
-	ret = qrv_add(call, wwl, qrv_int, wkd, "@KST", time(NULL));
+	ret = qrv_add(call, wwl, qrv_int, wkd, fullname ? fullname : "@KST", 0/*time(NULL)*/);
 	sw_qrv_sort(qrv);
 	return ret;
 }
@@ -1515,11 +1515,13 @@ void qrv_read_line(char *str){
 	if (regmatch(str, "^[0-9]{4}Z ([^ ]+)", &c1, &c2, NULL)==0){
 //		log_addf("KST '%s'", c2);
     
-        qi = qrv_get(qrv, c2);
-        if (qi){
-            qi->kst_time = time(NULL);
-        }
-        redraw_later();
+		if (strstr(str, "chat>") == NULL){
+			qi = qrv_get(qrv, c2);
+			if (qi){
+				qi->kst_time = time(NULL);
+			}
+			redraw_later();
+		}
     }
 
 	if (c1) g_free(c1);
