@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2022  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2023  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -24,10 +24,12 @@
 #include "map.h"
 #include "menu.h"
 #include "net.h"
+#include "oct8tor.h"
 #include "qrvdb.h"
 #include "qsodb.h"
 #include "rain.h"
 #include "rc.h"
+#include "rotar.h"
 #include "session.h"
 #include "sked.h"
 #include "ssbd.h"
@@ -178,7 +180,14 @@ int new_ctest(char *tdate){
 	maps_update_showwwls();
 	rain_reload();
 #endif  
-    
+
+#ifdef OCT8TOR
+	if (aband != NULL){
+		struct config_band *cb = get_config_band_by_bandchar(aband->bandchar);
+		oct8tor_send_band(goct8tor, cb->adifband);
+	}
+#endif
+
     return 0; 
 }
 
@@ -365,6 +374,7 @@ struct band *init_band(struct config_band *confb, GHashTable *opt_band, struct z
         if (!b->skedqrg) b->skedqrg=g_strdup("");
 		STORE_HASH_INT_DEF(b, qsomultb, 1);
 		if (b->qsomultb == 0) b->qsomultb = 1; // 3.12-3.13 set to 0 -> 0 points for QSO
+		STORE_HASH_DOUBLE_DEF(b, qrg, 0.0);
     }   
     b->bi = toupper(b->bandchar) - 'A';
     //dbg("%s bandname=%s\n",__FUNCTION__, b->bandname);  
@@ -570,6 +580,7 @@ void clear_tmpqsos(struct band *b, int set_defaults){
 #endif
 	}
 	ac_update_tmpctp("");
+	update_susp(b);
 }
 
 void default_rst_to_tmpqsos(struct band *b){
@@ -875,7 +886,14 @@ void activate_band(struct band *b){
         trig_resend_freq(trig);
     }
 #endif
+#ifdef OCT8TOR
+	struct config_band *cb = get_config_band_by_bandchar(b->bandchar);
+	oct8tor_send_band(goct8tor, cb->adifband);
+#endif
+	wkd_send_all(aband);
+
     redraw_later();
+    qrv_recalc_gst(qrv);
     maps_reload();
     chart_reload();
 #ifdef Z_HAVE_SDL
@@ -1044,7 +1062,7 @@ void add_qso_to_index(struct qso *q, int qsort_if_needed){
         }
     }
     z_ptr_array_add(ctest->allqsos, q);
-    if (strlen(q->exc) > ctest->exclen) ctest->exclen = strlen(q->exc);
+    if (q->exc != NULL && strlen(q->exc) > ctest->exclen) ctest->exclen = strlen(q->exc);
 
     qrv_set_wkd(qrv, q);
 }
@@ -1083,13 +1101,6 @@ int check_autosave(void){
     
     save_all_bands_txt(1);
 
-    if (cfg->as_floppy_aq){
-        if (ctest->as_floppy_qsonr <= 0){
-            save_all_bands_txt(2);
-            ctest->as_floppy_qsonr = cfg->as_floppy_aq;
-        }
-        ctest->as_floppy_qsonr--;
-    }
     return 0;
 }
 
