@@ -11,6 +11,7 @@
 #include "rain.h"
 
 #include <libzia.h>
+#include <zjson0.h>
 
 #include "header.h"
 
@@ -373,6 +374,7 @@ void free_rain(struct rain *rain){
     g_free(rain->colors);
     g_free(rain->bwcolors);
     g_string_free(rain->scplocs, TRUE);
+	g_ptr_array_free(rain->scps, TRUE);// with free func free_rain_scp
     g_free(rain);
 }
 
@@ -1241,7 +1243,11 @@ void rain_rainviewer_timer(void *arg){
 	provider->http = zhttp_init();
 	dbg("rainviewer %p url = '%s'\n", provider, url);
 	zhttp_get(provider->http, zsel, url, rain_rainviewer_downloaded_json, provider);
-	g_assert(provider->timer_id == 0);
+
+	if (provider->timer_id) {
+		zselect_timer_kill(zsel, provider->timer_id);
+		provider->timer_id = 0;
+	}
 	provider->timer_id = zselect_timer_new(zsel, 60 * 1000, rain_rainviewer_timer, provider);
 }
 
@@ -1256,24 +1262,28 @@ void rain_rainviewer_downloaded_json(struct zhttp *http){
 
 	if (http->state == ZHTTPST_ERROR && http->errorstr){
 		dbg("ERROR rain_rainviewer_downloaded_json: %s\n", http->errorstr);
-	}
-	dbg("rain_rainviewer_downloaded_json: got %d bytes\n", http->response->len - http->dataofs);
+	} else {
+		dbg("rain_rainviewer_downloaded_json: got %d bytes\n", http->response->len - http->dataofs);
 
-	if (http_is_content_type(http, "application/json")){
+		if (http_is_content_type(http, "application/json")) {
 
-		if (provider->rain->debug) zbinbuf_write_to_file(http->response, "_rainviewer.json", http->dataofs, http->response->len - http->dataofs);
+			if (provider->rain->debug) zbinbuf_write_to_file(http->response, "_rainviewer.json", http->dataofs, http->response->len - http->dataofs);
 
-		char *json = g_new(char, http->response->len + 1);
-		zbinbuf_getstr(http->response, http->dataofs, json, http->response->len + 1);
-		zhttp_free(http);
-		provider->http = NULL;
+			char* json = g_new(char, http->response->len + 1);
+			zbinbuf_getstr(http->response, http->dataofs, json, http->response->len + 1);
+			zhttp_free(http);
+			provider->http = NULL;
 
-		rain_rainviewer_process_json(provider, json);
-		g_free(json);
+			rain_rainviewer_process_json(provider, json);
+			g_free(json);
 
+		}
 	}
 
-	g_assert(provider->timer_id == 0);
+	if (provider->timer_id) {
+		zselect_timer_kill(zsel, provider->timer_id);
+		provider->timer_id = 0;
+	}
 	provider->timer_id = zselect_timer_new(zsel, 60 * 1000, rain_rainviewer_timer, provider);
 }
 
@@ -1284,7 +1294,7 @@ void rain_rainviewer_process_json(struct rain_provider *provider, const char *js
 	for (i = 0;; i++){
 		char key[246];
 		g_snprintf(key, sizeof(key), "radar.past.%d.time", i);
-		char *s = zjson_get_str(json, len, key);
+		char *s = zjson0_get_str(json, len, key);
 		if (s == NULL) break;
 
 		tt = (time_t)strtoll(s, NULL, 10);
@@ -1292,13 +1302,13 @@ void rain_rainviewer_process_json(struct rain_provider *provider, const char *js
 		//dbg("%d  tt=%lld\n", i, (long long)tt);
 
 		g_snprintf(key, sizeof(key), "radar.past.%d.path", i);
-		s = zjson_get_str(json, len, key);
+		s = zjson0_get_str(json, len, key);
 		if (s == NULL) break;
 
 		g_free(provider->rv_path);
 		provider->rv_path = s; 
 	}
-	char *host = zjson_get_str(json, len, "host");
+	char *host = zjson0_get_str(json, len, "host");
 	struct tm tm;
 	gmtime_r(&tt, &tm);
 	//dbg("rainviewer latest = %lld = %02d:%02dZ\n", (long long)tt,  tm.tm_hour, tm.tm_min);
