@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2022  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2023  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -397,7 +397,7 @@ void sdl_info(GString *gs){
     }
 #endif
 #ifdef Z_HAVE_SDL2
-	SDL_DisplayMode dm = { SDL_PIXELFORMAT_UNKNOWN, 0, 0, 0, 0 };
+	/*SDL_DisplayMode dm = { SDL_PIXELFORMAT_UNKNOWN, 0, 0, 0, 0 };
 	for (int di = 0; di < SDL_GetNumVideoDisplays(); di++){
 		g_string_append_printf(gs, "Display %d\n", di);
 
@@ -406,7 +406,7 @@ void sdl_info(GString *gs){
 			
 			g_string_append_printf(gs, "Mode %d: %dx%d, %d Hz, pixel format %s \n", mi, dm.w, dm.h, dm.refresh_rate, SDL_GetPixelFormatName(dm.format));
 		}
-	}
+	}*/
 #endif
 	g_string_append_printf(gs, "\n");
 }
@@ -435,9 +435,7 @@ int sdl_setvideomode(int w, int h, int init_x){
 #ifdef Z_HAVE_SDL1
 	flags |= SDL_RESIZABLE | SDL_HWPALETTE;
 #endif
-#ifdef Z_HAVE_SDL2
-	flags |= SDL_WINDOW_RESIZABLE;
-#endif
+
 	if (cfg->fullscreen){
 #ifdef Z_HAVE_SDL1
 		flags |= SDL_FULLSCREEN;
@@ -456,6 +454,9 @@ int sdl_setvideomode(int w, int h, int init_x){
 #endif
         }
     }else{
+#ifdef Z_HAVE_SDL2
+		flags |= SDL_WINDOW_RESIZABLE;
+#endif
 		sdl->window_w = w;
 		sdl->window_h = h;
 	}
@@ -494,6 +495,8 @@ int sdl_setvideomode(int w, int h, int init_x){
 			dbg("SDL_CreateWindow(%d,%d) failed\n", w, h);
 			return 1;
 		}
+	}else{
+		SDL_SetWindowFullscreen(zsdl->window, flags);
 	}
 	
 
@@ -601,6 +604,12 @@ int sdl_setvideomode(int w, int h, int init_x){
 #ifdef EVENTS_IN_SAME_THREAD
 	if (!sdl->event_timer_id) sdl->event_timer_id = zselect_timer_new(zsel, 100, sdl_event_timer, NULL);
 #endif
+
+#ifdef AUTOSAVE
+    if (init_x == 0 && gses != NULL) {
+        menu_save_rc(NULL);
+    }
+#endif
     return 0;
 }
 
@@ -1004,20 +1013,18 @@ gpointer sdl_event_thread(gpointer handle){
 
 
 #ifdef Z_HAVE_SDL1
-		switch(SDL_PeepEvents(&sev, 1, SDL_GETEVENT, SDL_ALLEVENTS)) {
+        int ne = SDL_PeepEvents(&sev, 1, SDL_GETEVENT, SDL_ALLEVENTS);
 #endif
 #ifdef Z_HAVE_SDL2
-		switch (SDL_PeepEvents(&sev, 1, SDL_GETEVENT, SDL_FIRSTEVENT, SDL_LASTEVENT)) {
+        int ne = SDL_PeepEvents(&sev, 1, SDL_GETEVENT, SDL_FIRSTEVENT, SDL_LASTEVENT);
 #endif
-		    case -1:
-                return GINT_TO_POINTER(-1);
-                break;
-		    case 0: 
-                SDL_Delay(50);
-                continue;
-		    case 1: 
-                break;
-		}
+        if (ne < 0) {
+            return GINT_TO_POINTER(-1);
+        }
+        if (ne == 0) {
+            SDL_Delay(50);
+            continue;
+        }
 
 
 #if 0
@@ -1041,102 +1048,118 @@ gpointer sdl_event_thread(gpointer handle){
         char type[10];
         strcpy(type, "?");
 
-        switch(sev.type){
+		switch(sev.type){
 #ifdef Z_HAVE_SDL2
-			case SDL_TEXTINPUT:
-                strcpy(type, "TEXTINPUT");
-				SDL_Keymod mod = SDL_GetModState();
-				dbg("TEXTINPUT: '%s'%s\n", sev.text.text, (mod & KMOD_ALT) ? " +ALT" : "");
-				if (mod & KMOD_ALT) break;
-				ev.ev = EV_KBD;
-				if (handle_unicode(&sev, &ev)) goto sendkeydown;
+		case SDL_TEXTINPUT:
+			strcpy(type, "TEXTINPUT");
+			SDL_Keymod mod = SDL_GetModState();
+			//dbg("TEXTINPUT: '%s'%s\n", sev.text.text, (mod & KMOD_ALT) ? " +ALT" : "");
+			if (mod & KMOD_ALT) break;
+			ev.ev = EV_KBD;
+			if (handle_unicode(&sev, &ev)) goto sendkeydown;
 
-				dbg("to je divny, SDL_TEXTINPUT '%s' nebyla zpracovana\n", sev.text.text);
-				break;
+			dbg("to je divny, SDL_TEXTINPUT '%s' nebyla zpracovana\n", sev.text.text);
+			break;
 #endif
-            case SDL_KEYDOWN:
-                strcpy(type, "KEYDOWN");
-#if 1
-				//if (sev.key.keysym.sym != SDLK_LCTRL && sev.key.keysym.sym != SDLK_RCTRL && sev.key.keysym.sym != SDLK_LALT && sev.key.keysym.sym != SDLK_RALT && sev.key.keysym.sym != SDLK_LSHIFT && sev.key.keysym.sym != SDLK_RSHIFT)
+		case SDL_KEYDOWN:
+			strcpy(type, "KEYDOWN");
+#if 0
+			//if (sev.key.keysym.sym != SDLK_LCTRL && sev.key.keysym.sym != SDLK_RCTRL && sev.key.keysym.sym != SDLK_LALT && sev.key.keysym.sym != SDLK_RALT && sev.key.keysym.sym != SDLK_LSHIFT && sev.key.keysym.sym != SDLK_RSHIFT)
+			if (sev.key.keysym.scancode != SDL_SCANCODE_LCTRL)
 				dbg("KEYDOWN: scan=%d sym=%d '%c' mod=%d (%s)   GetMods=(%s)\n", sev.key.keysym.scancode, sev.key.keysym.sym, sev.key.keysym.sym, sev.key.keysym.mod, dumpmods(&sev.key.keysym), dumpmods(NULL));
 #endif                
 
-				if (cfg->altsyms){
-                    oldaltsym = sdl->altsym;
-                    if (sev.key.keysym.sym == SDLK_LALT || sev.key.keysym.sym == SDLK_RALT){
-                        sdl->altsym = 1;
-                        //dbg("sdl->altsym=1\n");
-                    }else{
-                        sdl->altsym = 0;
-                        //dbg("sdl->altsym=0\n");
-                    }
-                }    
-                
-                ev.ev=EV_KBD;
-                switch(sev.key.keysym.sym){
-                    case SDLK_BACKSPACE:    k=sk=KBD_BS;         goto handle;                                            
-                    case SDLK_TAB:          k=sk=KBD_TAB;        goto handle;
-                    case SDLK_RETURN:                
-                    case SDLK_KP_ENTER:     k=sk=KBD_ENTER;      goto handle;
-					case SDLK_PAUSE:		/*fprintf(stderr, "PAUSE\n");*/      goto handle;
-                    case SDLK_ESCAPE:       k=sk=KBD_ESC;        goto handle;
+			if (cfg->altsyms){
+				oldaltsym = sdl->altsym;
+				if (sev.key.keysym.sym == SDLK_LALT || sev.key.keysym.sym == SDLK_RALT){
+					sdl->altsym = 1;
+					//dbg("sdl->altsym=1\n");
+				}else{
+					sdl->altsym = 0;
+					//dbg("sdl->altsym=0\n");
+				}
+			}
 
-                    case SDLK_UP: 		    k=sk=KBD_UP;         goto handle;
-                    case SDLK_DOWN: 	    k=sk=KBD_DOWN;       goto handle;
-                    case SDLK_RIGHT: 	    k=sk=KBD_RIGHT;      goto handle;
-                    case SDLK_LEFT: 	    k=sk=KBD_LEFT;       goto handle;
-                    case SDLK_INSERT: 	    k=sk=KBD_INS;        goto handle;
-                    case SDLK_DELETE:       k=sk=KBD_DEL;        goto handle;
-                    case SDLK_HOME: 	    k=sk=KBD_HOME;       goto handle;
-                    case SDLK_END: 		    k=sk=KBD_END;        goto handle;
-                    case SDLK_PAGEUP: 	    k=sk=KBD_PGUP;       goto handle;
-                    case SDLK_PAGEDOWN:     k=sk=KBD_PGDN;       goto handle;
-                    case SDLK_F1: 		    k=sk=KBD_F1;         goto handle;
-                    case SDLK_F2: 		    k=sk=KBD_F2;         goto handle;
-                    case SDLK_F3: 		    k=sk=KBD_F3;         goto handle;
-                    case SDLK_F4: 		    k=sk=KBD_F4;         goto handle;
-                    case SDLK_F5: 		    k=sk=KBD_F5;         goto handle;
-                    case SDLK_F6: 		    k=sk=KBD_F6;         goto handle;
-                    case SDLK_F7: 		    k=sk=KBD_F7;         goto handle;
-                    case SDLK_F8: 		    k=sk=KBD_F8;         goto handle;
-                    case SDLK_F9: 		    k=sk=KBD_F9;         goto handle;
-                    case SDLK_F10: 		    k=sk=KBD_F10;        goto handle;
-                    case SDLK_F11: 		    k=sk=KBD_F11;        goto handle;
-                    case SDLK_F12: 		    k=sk=KBD_F12;        goto handle;
+			ev.ev=EV_KBD;
+			switch(sev.key.keysym.sym){
+
+			case SDLK_BACKSPACE:    k= sk = KBD_BS;         goto handle;
+			case SDLK_TAB:          k = sk = KBD_TAB;        goto handle;
+			case SDLK_RETURN:
+			case SDLK_KP_ENTER:     k = sk = KBD_ENTER;      goto handle;
+			case SDLK_PAUSE:		/*fprintf(stderr, "PAUSE\n");*/      goto handle;
+			case SDLK_ESCAPE:       k = sk = KBD_ESC;        goto handle;
+
+			case SDLK_UP: 		    k = sk = KBD_UP;         goto handle;
+			case SDLK_DOWN: 	    k = sk = KBD_DOWN;       goto handle;
+			case SDLK_RIGHT: 	    k = sk = KBD_RIGHT;      goto handle;
+			case SDLK_LEFT: 	    k = sk = KBD_LEFT;       goto handle;
+			case SDLK_INSERT: 	    k = sk = KBD_INS;        goto handle;
+			case SDLK_DELETE:       k = sk = KBD_DEL;        goto handle;
+			case SDLK_HOME: 	    k = sk = KBD_HOME;       goto handle;
+			case SDLK_END: 		    k = sk = KBD_END;        goto handle;
+			case SDLK_PAGEUP: 	    k = sk = KBD_PGUP;       goto handle;
+			case SDLK_PAGEDOWN:     k = sk = KBD_PGDN;       goto handle;
+			case SDLK_F1: 		    k = sk = KBD_F1;         goto handle;
+			case SDLK_F2: 		    k = sk = KBD_F2;         goto handle;
+			case SDLK_F3: 		    k = sk = KBD_F3;         goto handle;
+			case SDLK_F4: 		    k = sk = KBD_F4;         goto handle;
+			case SDLK_F5: 		    k = sk = KBD_F5;         goto handle;
+			case SDLK_F6: 		    k = sk = KBD_F6;         goto handle;
+			case SDLK_F7: 		    k = sk = KBD_F7;         goto handle;
+			case SDLK_F8: 		    k = sk = KBD_F8;         goto handle;
+			case SDLK_F9: 		    k = sk = KBD_F9;         goto handle;
+			case SDLK_F10: 		    k = sk = KBD_F10;        goto handle;
+			case SDLK_F11: 		    k = sk = KBD_F11;        goto handle;
+			case SDLK_F12: 		    k = sk = KBD_F12;        goto handle;
 #ifdef Z_HAVE_SDL1
-                    case SDLK_KP0:          k=KBD_INS;   sk='0'; goto handle;
-                    case SDLK_KP1:          k=KBD_END;   sk='1'; goto handle;
-                    case SDLK_KP2:          k=KBD_DOWN;  sk='2'; goto handle;
-                    case SDLK_KP3:          k=KBD_PGDN;  sk='3'; goto handle;
-                    case SDLK_KP4:          k=KBD_LEFT;  sk='4'; goto handle;
-                    case SDLK_KP5:          k=KBD_DOWN;  sk='5'; goto handle;
-                    case SDLK_KP6:          k=KBD_RIGHT; sk='6'; goto handle;
-                    case SDLK_KP7:          k=KBD_HOME;  sk='7'; goto handle;
-                    case SDLK_KP8:          k=KBD_UP;    sk='8'; goto handle;
-                    case SDLK_KP9:          k=KBD_PGUP;  sk='9'; goto handle;
+			case SDLK_KP0:          k=KBD_INS;   sk='0'; goto handle;
+			case SDLK_KP1:          k=KBD_END;   sk='1'; goto handle;
+			case SDLK_KP2:          k=KBD_DOWN;  sk='2'; goto handle;
+			case SDLK_KP3:          k=KBD_PGDN;  sk='3'; goto handle;
+			case SDLK_KP4:          k=KBD_LEFT;  sk='4'; goto handle;
+			case SDLK_KP5:          k=KBD_DOWN;  sk='5'; goto handle;
+			case SDLK_KP6:          k=KBD_RIGHT; sk='6'; goto handle;
+			case SDLK_KP7:          k=KBD_HOME;  sk='7'; goto handle;
+			case SDLK_KP8:          k=KBD_UP;    sk='8'; goto handle;
+			case SDLK_KP9:          k=KBD_PGUP;  sk='9'; goto handle;
 #endif
 #ifdef Z_HAVE_SDL2 // special handled by TEXTINPUT, number here
-					case SDLK_KP_0:          k = KBD_INS;   goto handle;
-					case SDLK_KP_1:          k = KBD_END;   goto handle;
-					case SDLK_KP_2:          k = KBD_DOWN;  goto handle;
-					case SDLK_KP_3:          k = KBD_PGDN;  goto handle;
-					case SDLK_KP_4:          k = KBD_LEFT;  goto handle;
-					case SDLK_KP_5:          k = KBD_DOWN;  goto handle;
-					case SDLK_KP_6:          k = KBD_RIGHT; goto handle;
-					case SDLK_KP_7:          k = KBD_HOME;  goto handle;
-					case SDLK_KP_8:          k = KBD_UP;    goto handle;
-					case SDLK_KP_9:          k = KBD_PGUP;  goto handle;
+			case SDLK_KP_0:          k = KBD_INS;   goto handle;
+			case SDLK_KP_1:          k = KBD_END;   goto handle;
+			case SDLK_KP_2:          k = KBD_DOWN;  goto handle;
+			case SDLK_KP_3:          k = KBD_PGDN;  goto handle;
+			case SDLK_KP_4:          k = KBD_LEFT;  goto handle;
+			case SDLK_KP_5:          k = KBD_DOWN;  goto handle;
+			case SDLK_KP_6:          k = KBD_RIGHT; goto handle;
+			case SDLK_KP_7:          k = KBD_HOME;  goto handle;
+			case SDLK_KP_8:          k = KBD_UP;    goto handle;
+			case SDLK_KP_9:          k = KBD_PGUP;  goto handle;
 #endif
 #ifdef Z_HAVE_SDL1
-					case SDLK_KP_PERIOD:     k = KBD_DEL;   sk='.'; goto handle; // jeste je SDLK_PERIOD
-					case SDLK_KP_DIVIDE:     k = sk = '/';          goto handle;
-					case SDLK_KP_MULTIPLY:   k = sk = '*';          goto handle;
-                    case SDLK_KP_MINUS: 	 k = sk = '-';          goto handle;
-                    case SDLK_KP_PLUS: 	     k = sk = '+';          goto handle;
+			case SDLK_KP_PERIOD:     k = KBD_DEL;   sk='.'; goto handle; // jeste je SDLK_PERIOD
+			case SDLK_KP_DIVIDE:     k = sk = '/';          goto handle;
+			case SDLK_KP_MULTIPLY:   k = sk = '*';          goto handle;
+			case SDLK_KP_MINUS: 	 k = sk = '-';          goto handle;
+			case SDLK_KP_PLUS: 	     k = sk = '+';          goto handle;
+#endif
+			case SDLK_BACKQUOTE:    k = '`'; sk = '~';   goto handle;
+			default: break;
+			}
+#ifdef Z_HAVE_SDL2
+			if ((sev.key.keysym.mod & (KMOD_CTRL | KMOD_ALT)) != 0){
+				switch (sev.key.keysym.scancode){
+					case SDL_SCANCODE_KP_PERIOD:	k = KBD_DEL;   sk = '.'; goto handle;
+					case SDL_SCANCODE_KP_DIVIDE:	k = sk = '/';            goto handle;
+					case SDL_SCANCODE_KP_MULTIPLY:	k = sk = '*';            goto handle;
+					case SDL_SCANCODE_KP_MINUS:		k = sk = '-';            goto handle;
+					case SDL_SCANCODE_KP_PLUS:		k = sk = '+';            goto handle;
+                    default: 
+                        break;
+				}
+			}
+			// else: without ctrl/alt keys are sent as TEXTINPUT
 #endif
-					case SDLK_BACKQUOTE:    k = '`'; sk = '~';   goto handle;
-                    default: break;
-                }
 
                 int skip_unicode = 0;
                 if (sev.key.keysym.sym >= SDLK_0 && sev.key.keysym.sym <= SDLK_9 
@@ -1371,7 +1394,7 @@ gpointer sdl_event_thread(gpointer handle){
 					ev.my = sdl->resize_h = sev.window.data2;
 					ev.x = ev.mx / FONT_W;
 					ev.y = ev.my / FONT_H;
-					dbg("SDL_WINDOWEVENT_RESIZED %d w=%d\n", sdl->resizeevents, sdl->resize_w);
+					//dbg("SDL_WINDOWEVENT_RESIZED %d w=%d\n", sdl->resizeevents, sdl->resize_w);
 					if (sdl->resizeevents > 0) continue;
 					goto send;
 				}
@@ -1387,7 +1410,7 @@ gpointer sdl_event_thread(gpointer handle){
         continue;
 sendkeydown:        
         //dbg("cfg->altsyms=%d sdl->altsym=%d ev->x=%c\n", cfg->altsyms, sdl->altsym, ev.x);
-        dbg("SEND %s\n", type);
+        //dbg("SEND %s\n", type);
         if (oldaltsym){
             int oldk = ev.x;
             switch (ev.x){
@@ -1621,7 +1644,7 @@ void progress(char *m, ...){
 	if (!m) {
 		if (term) memset(term->last_screen, 0, term->x * term->y * sizeof(unsigned));
 		return;
-	}
+	}				
     va_start(l, m);
     c = g_strdup_vprintf(m, l);
     dbg("%s\n", c);
